@@ -35,6 +35,17 @@
  * the GnomeVFS library. Common protocols are 'file', 'http', 'ftp', or 'smb'.
  * </para>
  * <para>
+ * In case the element-gnomevfssrc::iradio-mode property is set and the
+ * location is a http resource, gnomevfssrc will send special icecast http
+ * headers to the server to request additional icecast metainformation. If
+ * the server is not an icecast server, it will display the same behaviour
+ * as if the element-gnomevfssrc::iradio-mode property was not set. However,
+ * if the server is in fact an icecast server, gnomevfssrc will output
+ * data with a media type of application/x-icy, in which case you will
+ * need to use the #ICYDemux element as follow-up element to extract
+ * the icecast meta data and to determine the underlying media type.
+ * </para>
+ * <para>
  * Example pipeline:
  * <programlisting>
  * gst-launch -v gnomevfssrc location=file:///home/joe/foo.xyz ! fakesink
@@ -328,7 +339,7 @@ gst_gnome_vfs_src_finalize (GObject * object)
  * URI interface support.
  */
 
-static guint
+static GstURIType
 gst_gnome_vfs_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -843,14 +854,15 @@ open_failed:
     gst_gnome_vfs_src_pop_callbacks (src);
 
     if (res == GNOME_VFS_ERROR_NOT_FOUND ||
+        res == GNOME_VFS_ERROR_HOST_NOT_FOUND ||
         res == GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE) {
       GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s",
-              filename, gnome_vfs_result_to_string (res)));
+          ("Could not open vfs file \"%s\" for reading: %s (%d)",
+              filename, gnome_vfs_result_to_string (res), res));
     } else {
       GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-          ("Could not open vfs file \"%s\" for reading: %s",
-              filename, gnome_vfs_result_to_string (res)));
+          ("Could not open vfs file \"%s\" for reading: %s (%d)",
+              filename, gnome_vfs_result_to_string (res), res));
     }
     g_free (filename);
     return FALSE;
