@@ -30,7 +30,7 @@
 #define GST_TYPE_VISUAL (gst_visual_get_type())
 #define GST_IS_VISUAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VISUAL))
 #define GST_VISUAL(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VISUAL,GstVisual))
-#define GST_IS_VISUAL_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VISUAL))
+#define GST_IS_VISUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VISUAL))
 #define GST_VISUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VISUAL,GstVisualClass))
 #define GST_VISUAL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_VISUAL, GstVisualClass))
 
@@ -51,7 +51,7 @@ struct _GstVisual
   GstSegment segment;
 
   /* libvisual stuff */
-  VisAudio audio;
+  VisAudio *audio;
   VisVideo *video;
   VisActor *actor;
 
@@ -109,8 +109,13 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
         "width = (int) 16, "
         "depth = (int) 16, "
         "endianness = (int) BYTE_ORDER, "
-        "signed = (boolean) TRUE, "
-        "channels = (int) { 1, 2 }, " "rate = (int) [ 1000, MAX ]")
+        "signed = (boolean) TRUE, " "channels = (int) { 1, 2 }, "
+#if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+        "rate = (int) { 8000, 11250, 22500, 32000, 44100, 48000, 96000 }"
+#else
+        "rate = (int) [ 1000, MAX ]"
+#endif
+    )
     );
 
 
@@ -228,6 +233,10 @@ gst_visual_clear_actors (GstVisual * visual)
     visual_object_unref (VISUAL_OBJECT (visual->video));
     visual->video = NULL;
   }
+  if (visual->audio) {
+    visual_object_unref (VISUAL_OBJECT (visual->audio));
+    visual->audio = NULL;
+  }
 }
 
 static void
@@ -308,7 +317,7 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   GstStructure *structure;
-  gint depth;
+  gint depth, pitch;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -327,12 +336,12 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
   visual_video_set_depth (visual->video,
       visual_video_depth_enum_from_value (depth));
   visual_video_set_dimension (visual->video, visual->width, visual->height);
+  pitch = GST_ROUND_UP_4 (visual->width * visual->video->bpp);
+  visual_video_set_pitch (visual->video, pitch);
   visual_actor_video_negotiate (visual->actor, 0, FALSE, FALSE);
 
   /* precalc some values */
-  visual->outsize =
-      visual->video->height * GST_ROUND_UP_4 (visual->video->width) *
-      visual->video->bpp;
+  visual->outsize = visual->video->height * pitch;
   visual->spf =
       gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
   visual->duration =
@@ -411,7 +420,6 @@ no_format:
     GST_ELEMENT_ERROR (visual, STREAM, FORMAT, (NULL),
         ("could not negotiate output format"));
     gst_caps_unref (intersect);
-    gst_caps_unref (othercaps);
     return FALSE;
   }
 }
@@ -612,17 +620,78 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     data =
         (const guint16 *) gst_adapter_peek (visual->adapter, 512 * visual->bps);
 
+#if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+    {
+      VisBuffer *lbuf, *rbuf;
+      guint16 ldata[512], rdata[512];
+      VisAudioSampleRateType rate;
+
+      lbuf = visual_buffer_new_with_buffer (ldata, sizeof (ldata), NULL);
+      rbuf = visual_buffer_new_with_buffer (rdata, sizeof (rdata), NULL);
+
+      if (visual->channels == 2) {
+        for (i = 0; i < 512; i++) {
+          ldata[i] = *data++;
+          rdata[i] = *data++;
+        }
+      } else {
+        for (i = 0; i < 512; i++) {
+          ldata[i] = *data;
+          rdata[i] = *data++;
+        }
+      }
+
+      switch (visual->rate) {
+        case 8000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_8000;
+          break;
+        case 11250:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_11250;
+          break;
+        case 22500:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_22500;
+          break;
+        case 32000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_32000;
+          break;
+        case 44100:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_44100;
+          break;
+        case 48000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_48000;
+          break;
+        case 96000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_96000;
+          break;
+        default:
+          g_assert_not_reached ();
+          break;
+      }
+
+      visual_audio_samplepool_input_channel (visual->audio->samplepool,
+          lbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_LEFT);
+      visual_audio_samplepool_input_channel (visual->audio->samplepool,
+          rbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_RIGHT);
+
+      visual_object_unref (VISUAL_OBJECT (lbuf));
+      visual_object_unref (VISUAL_OBJECT (rbuf));
+
+    }
+#else
     if (visual->channels == 2) {
       for (i = 0; i < 512; i++) {
-        visual->audio.plugpcm[0][i] = *data++;
-        visual->audio.plugpcm[1][i] = *data++;
+        visual->audio->plugpcm[0][i] = *data++;
+        visual->audio->plugpcm[1][i] = *data++;
       }
     } else {
       for (i = 0; i < 512; i++) {
-        visual->audio.plugpcm[0][i] = *data;
-        visual->audio.plugpcm[1][i] = *data++;
+        visual->audio->plugpcm[0][i] = *data;
+        visual->audio->plugpcm[1][i] = *data++;
       }
     }
+#endif
 
     /* alloc a buffer if we don't have one yet, this happens
      * when we pushed a buffer in this while loop before */
@@ -633,9 +702,10 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
       }
     }
     visual_video_set_buffer (visual->video, GST_BUFFER_DATA (outbuf));
-    visual_audio_analyze (&visual->audio);
-    visual_actor_run (visual->actor, &visual->audio);
+    visual_audio_analyze (visual->audio);
+    visual_actor_run (visual->actor, visual->audio);
     visual_video_set_buffer (visual->video, NULL);
+    GST_DEBUG_OBJECT (visual, "rendered one frame");
 
     GST_BUFFER_TIMESTAMP (outbuf) = visual->next_ts;
     GST_BUFFER_DURATION (outbuf) = visual->duration;
@@ -682,7 +752,7 @@ gst_visual_change_state (GstElement * element, GstStateChange transition)
           visual_actor_new (GST_VISUAL_GET_CLASS (visual)->plugin->info->
           plugname);
       visual->video = visual_video_new ();
-
+      visual->audio = visual_audio_new ();
       /* can't have a play without actors */
       if (!visual->actor || !visual->video)
         goto no_actors;
