@@ -237,7 +237,7 @@ gst_vorbis_enc_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static GstAudioChannelPosition vorbischannelpositions[][6] = {
+static const GstAudioChannelPosition vorbischannelpositions[][6] = {
   {                             /* Mono */
       GST_AUDIO_CHANNEL_POSITION_FRONT_MONO},
   {                             /* Stereo */
@@ -637,6 +637,7 @@ gst_vorbis_enc_metadata_set1 (const GstTagList * list, const gchar * tag,
     const gchar *vc_string = (const gchar *) l->data;
     gchar *key = NULL, *val = NULL;
 
+    GST_LOG_OBJECT (vorbisenc, "vorbis comment: %s", vc_string);
     if (gst_tag_parse_extended_comment (vc_string, &key, NULL, &val, TRUE)) {
       vorbis_comment_add_tag (&enc->vc, key, val);
       g_free (key);
@@ -786,7 +787,7 @@ gst_vorbis_enc_setup (GstVorbisEnc * vorbisenc)
             max_bitrate, vorbisenc->bitrate, min_bitrate) != 0) {
       GST_ERROR_OBJECT (vorbisenc,
           "vorbis_encode_setup_managed "
-          "(c %d, rate %d, max br %ld, br %ld, min br %ld) failed",
+          "(c %d, rate %d, max br %ld, br %d, min br %ld) failed",
           vorbisenc->channels, vorbisenc->frequency, max_bitrate,
           vorbisenc->bitrate, min_bitrate);
       vorbis_info_clear (&vorbisenc->vi);
@@ -1011,8 +1012,8 @@ gst_vorbis_enc_buffer_check_discontinuous (GstVorbisEnc * vorbisenc,
 
     if ((GstClockTimeDiff) (GST_BUFFER_TIMESTAMP (buffer) -
             vorbisenc->expected_ts) > halfsample) {
-      GST_DEBUG_OBJECT (vorbisenc, "Expected TS % " GST_TIME_FORMAT
-          ", buffer TS % " GST_TIME_FORMAT,
+      GST_DEBUG_OBJECT (vorbisenc, "Expected TS %" GST_TIME_FORMAT
+          ", buffer TS %" GST_TIME_FORMAT,
           GST_TIME_ARGS (vorbisenc->expected_ts),
           GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
       ret = TRUE;
