@@ -55,6 +55,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   GstCaps *caps = NULL;
   GstBuffer *palette = NULL;
 
+  GST_DEBUG ("video fourcc %" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (codec_fcc));
+
   switch (codec_fcc) {
     case GST_MAKE_FOURCC ('D', 'I', 'B', ' '):{
       gint bpp = (strf && strf->bit_cnt != 0) ? strf->bit_cnt : 8;
@@ -174,58 +176,68 @@ gst_riff_create_video_caps (guint32 codec_fcc,
     case GST_MAKE_FOURCC ('h', '2', '6', '3'):
     case GST_MAKE_FOURCC ('i', '2', '6', '3'):
     case GST_MAKE_FOURCC ('U', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "itu", NULL);
       if (codec_name)
         *codec_name = g_strdup ("ITU H.26n");
       break;
 
     case GST_MAKE_FOURCC ('L', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      /* http://www.leadcodecs.com/Codecs/LEAD-H263.htm */
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "lead", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Lead H.263");
       break;
 
     case GST_MAKE_FOURCC ('M', '2', '6', '3'):
     case GST_MAKE_FOURCC ('m', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "microsoft", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft H.263");
       break;
 
     case GST_MAKE_FOURCC ('V', 'D', 'O', 'W'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "vdolive", NULL);
       if (codec_name)
         *codec_name = g_strdup ("VDOLive");
       break;
 
     case GST_MAKE_FOURCC ('V', 'I', 'V', 'O'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "vivo", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Vivo H.263");
       break;
 
     case GST_MAKE_FOURCC ('x', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "xirlink", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Xirlink H.263");
       break;
 
       /* apparently not standard H.263...? */
     case GST_MAKE_FOURCC ('I', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-intel-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-intel-h263",
+          "variant", G_TYPE_STRING, "intel", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Intel H.263");
       break;
 
     case GST_MAKE_FOURCC ('H', '2', '6', '4'):
     case GST_MAKE_FOURCC ('h', '2', '6', '4'):
-      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      caps = gst_caps_new_simple ("video/x-h264",
+          "variant", G_TYPE_STRING, "itu", NULL);
       if (codec_name)
         *codec_name = g_strdup ("ITU H.264");
       break;
 
     case GST_MAKE_FOURCC ('V', 'S', 'S', 'H'):
-      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      caps = gst_caps_new_simple ("video/x-h264",
+          "variant", G_TYPE_STRING, "videosoft", NULL);
       if (codec_name)
         *codec_name = g_strdup ("VideoSoft H.264");
       break;
@@ -509,11 +521,20 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('T', 'S', 'C', 'C'):
-    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):
-      caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):{
+      if (strf) {
+        gint depth = (strf->bit_cnt != 0) ? (gint) strf->bit_cnt : 24;
+
+        caps = gst_caps_new_simple ("video/x-camtasia", "depth", G_TYPE_INT,
+            depth, NULL);
+      } else {
+        /* template caps */
+        caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+      }
       if (codec_name)
         *codec_name = g_strdup ("TechSmith Camtasia");
       break;
+    }
 
     case GST_MAKE_FOURCC ('V', 'C', 'R', '1'):
       caps = gst_caps_new_simple ("video/x-ati-vcr",
@@ -565,6 +586,12 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("VMWare NC Video");
       break;
 
+    case GST_MAKE_FOURCC ('d', 'r', 'a', 'c'):
+      caps = gst_caps_new_simple ("video/x-dirac", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Dirac");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -613,12 +640,16 @@ gst_riff_create_video_caps (guint32 codec_fcc,
           GST_BUFFER_SIZE (palette));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      gint n;
-      guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
+      {
+        guint8 *data = GST_BUFFER_DATA (copy);
+        gint n;
 
-      /* own endianness */
-      for (n = 0; n < num_colors; n++)
-        data[n] = GUINT32_FROM_LE (data[n]);
+        /* own endianness */
+        for (n = 0; n < num_colors; n++) {
+          GST_WRITE_UINT32_BE (data, GST_READ_UINT32_LE (data));
+          data += sizeof (guint32);
+        }
+      }
 #endif
       gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
       gst_buffer_unref (copy);
@@ -720,27 +751,15 @@ gst_riff_create_audio_caps (guint16 codec_id,
   gint channels_max = 2;
 
   switch (codec_id) {
-    case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
-      caps = gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG-1 layer 3");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
-      caps = gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG-1 layer 2");
-      break;
-
     case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM */
       if (strf != NULL) {
         gint ba = strf->blockalign;
         gint ch = strf->channels;
         gint ws = strf->size;
 
-        caps = gst_caps_new_simple ("audio/x-raw-int", "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, "channels", G_TYPE_INT, ch,   /* needed for _add_layout() */
+        caps = gst_caps_new_simple ("audio/x-raw-int",
+            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            "channels", G_TYPE_INT, ch,
             "width", G_TYPE_INT, (int) (ba * 8 / ch),
             "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, ws != 8, NULL);
 
@@ -793,6 +812,70 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
+    case GST_RIFF_WAVE_FORMAT_IBM_CVSD:
+      goto unknown;
+
+    case GST_RIFF_WAVE_FORMAT_ALAW:
+      if (strf != NULL) {
+        if (strf->size != 8) {
+          GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
+              strf->size);
+          strf->size = 8;
+          strf->av_bps = 8;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+        if (strf->av_bps == 0 || strf->blockalign == 0) {
+          GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+              strf->av_bps, strf->blockalign);
+          strf->av_bps = strf->size;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+      }
+      caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("A-law audio");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_WMS:
+      caps = gst_caps_new_simple ("audio/x-wms", NULL);
+      if (strf != NULL) {
+        gst_caps_set_simple (caps,
+            "bitrate", G_TYPE_INT, strf->av_bps * 8,
+            "width", G_TYPE_INT, strf->size,
+            "depth", G_TYPE_INT, strf->size, NULL);
+      } else {
+        gst_caps_set_simple (caps,
+            "bitrate", GST_TYPE_INT_RANGE, 0, G_MAXINT, NULL);
+      }
+      if (codec_name)
+        *codec_name = g_strdup ("Windows Media Audio Speech");
+      block_align = TRUE;
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_MULAW:
+      if (strf != NULL) {
+        if (strf->size != 8) {
+          GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
+              strf->size);
+          strf->size = 8;
+          strf->av_bps = 8;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+        if (strf->av_bps == 0 || strf->blockalign == 0) {
+          GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+              strf->av_bps, strf->blockalign);
+          strf->av_bps = strf->size;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+      }
+      caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Mu-law audio");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_OKI_ADPCM:
+      goto unknown;
+
     case GST_RIFF_WAVE_FORMAT_DVI_ADPCM:
       caps = gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, "dvi", NULL);
@@ -801,42 +884,25 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
-    case GST_RIFF_WAVE_FORMAT_MULAW:
-      if (strf != NULL && strf->size != 8) {
-        GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
-            strf->size);
-        strf->size = 8;
-        strf->av_bps = 8;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
-        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
-            strf->av_bps, strf->blockalign);
-        strf->av_bps = strf->size;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+    case GST_RIFF_WAVE_FORMAT_GSM610:
+    case GST_RIFF_WAVE_FORMAT_MSN:
+      caps = gst_caps_new_simple ("audio/ms-gsm", NULL);
       if (codec_name)
-        *codec_name = g_strdup ("Mu-law audio");
+        *codec_name = g_strdup ("MS GSM audio");
       break;
 
-    case GST_RIFF_WAVE_FORMAT_ALAW:
-      if (strf != NULL && strf->size != 8) {
-        GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
-            strf->size);
-        strf->size = 8;
-        strf->av_bps = 8;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
-        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
-            strf->av_bps, strf->blockalign);
-        strf->av_bps = strf->size;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+    case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
       if (codec_name)
-        *codec_name = g_strdup ("A-law audio");
+        *codec_name = g_strdup ("MPEG-1 layer 2");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG-1 layer 3");
       break;
 
     case GST_RIFF_WAVE_FORMAT_VORBIS1: /* ogg/vorbis mode 1 */
@@ -862,6 +928,15 @@ gst_riff_create_audio_caps (guint16 codec_id,
       if (codec_name)
         *codec_name = g_strdup ("DTS audio");
       break;
+    case GST_RIFF_WAVE_FORMAT_AAC:
+    {
+      channels_max = 8;
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 4, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG-4 AAC audio");
+      break;
+    }
     case GST_RIFF_WAVE_FORMAT_WMAV1:
     case GST_RIFF_WAVE_FORMAT_WMAV2:
     case GST_RIFF_WAVE_FORMAT_WMAV3:
@@ -869,7 +944,6 @@ gst_riff_create_audio_caps (guint16 codec_id,
       gint version = (codec_id - GST_RIFF_WAVE_FORMAT_WMAV1) + 1;
 
       channels_max = 6;
-
       block_align = TRUE;
 
       caps = gst_caps_new_simple ("audio/x-wma",
@@ -888,6 +962,11 @@ gst_riff_create_audio_caps (guint16 codec_id,
       }
       break;
     }
+    case GST_RIFF_WAVE_FORMAT_WMAV3_L:
+    {
+      /* WMA Version 9 Lossless */
+      goto unknown;
+    }
     case GST_RIFF_WAVE_FORMAT_SONY_ATRAC3:
       caps = gst_caps_new_simple ("audio/x-vnd.sony.atrac3", NULL);
       if (codec_name)
@@ -962,6 +1041,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
           subformat_guid[1] == 0x41783f83 &&
           subformat_guid[2] == 0xf0006596 && subformat_guid[3] == 0xe59262bf) {
         caps = gst_caps_new_simple ("application/x-ogg-avi", NULL);
+        if (codec_name)
+          *codec_name = g_strdup ("Ogg-AVI");
       }
 
       if (caps == NULL) {
@@ -970,7 +1051,6 @@ gst_riff_create_audio_caps (guint16 codec_id,
       }
       break;
     }
-
       /* can anything decode these? pitfdll? */
     case GST_RIFF_WAVE_FORMAT_VOXWARE:
     case GST_RIFF_WAVE_FORMAT_VOXWARE_BYTE_ALIGNED:
@@ -992,6 +1072,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
     }
     default:
+    unknown:
       GST_WARNING ("Unknown audio tag 0x%04x", codec_id);
       return NULL;
   }
@@ -1074,12 +1155,15 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
     GST_MAKE_FOURCC ('H', '2', '6', '3'),
+    GST_MAKE_FOURCC ('H', '2', '6', '4'),
     GST_MAKE_FOURCC ('H', 'F', 'Y', 'U'),
     GST_MAKE_FOURCC ('I', '2', '6', '3'),
     GST_MAKE_FOURCC ('I', '4', '2', '0'),
     GST_MAKE_FOURCC ('I', 'V', '3', '2'),
     GST_MAKE_FOURCC ('I', 'V', '4', '1'),
     GST_MAKE_FOURCC ('I', 'V', '5', '0'),
+    GST_MAKE_FOURCC ('L', '2', '6', '3'),
+    GST_MAKE_FOURCC ('M', '2', '6', '3'),
     GST_MAKE_FOURCC ('M', '4', 'S', '2'),
     GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
     GST_MAKE_FOURCC ('M', 'P', '4', '2'),
@@ -1097,8 +1181,11 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('U', 'L', 'T', 'I'),
     GST_MAKE_FOURCC ('V', 'C', 'R', '1'),
     GST_MAKE_FOURCC ('V', 'C', 'R', '2'),
+    GST_MAKE_FOURCC ('V', 'D', 'O', 'W'),
+    GST_MAKE_FOURCC ('V', 'I', 'V', 'O'),
     GST_MAKE_FOURCC ('V', 'M', 'n', 'c'),
     GST_MAKE_FOURCC ('V', 'P', '3', ' '),
+    GST_MAKE_FOURCC ('V', 'S', 'S', 'H'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '1'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '2'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '3'),
@@ -1109,7 +1196,8 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('Z', 'L', 'I', 'B'),
     GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
     GST_MAKE_FOURCC ('h', '2', '6', '4'),
-    GST_MAKE_FOURCC ('m', 's', 'v', 'c')
+    GST_MAKE_FOURCC ('m', 's', 'v', 'c'),
+    GST_MAKE_FOURCC ('x', '2', '6', '3')
         /* FILL ME */
   };
   guint i;
@@ -1129,14 +1217,17 @@ GstCaps *
 gst_riff_create_audio_template_caps (void)
 {
   static const guint16 tags[] = {
+    GST_RIFF_WAVE_FORMAT_GSM610,
     GST_RIFF_WAVE_FORMAT_MPEGL3,
     GST_RIFF_WAVE_FORMAT_MPEGL12,
     GST_RIFF_WAVE_FORMAT_PCM,
     GST_RIFF_WAVE_FORMAT_VORBIS1,
     GST_RIFF_WAVE_FORMAT_A52,
     GST_RIFF_WAVE_FORMAT_DTS,
+    GST_RIFF_WAVE_FORMAT_AAC,
     GST_RIFF_WAVE_FORMAT_ALAW,
     GST_RIFF_WAVE_FORMAT_MULAW,
+    GST_RIFF_WAVE_FORMAT_WMS,
     GST_RIFF_WAVE_FORMAT_ADPCM,
     GST_RIFF_WAVE_FORMAT_DVI_ADPCM,
     GST_RIFF_WAVE_FORMAT_WMAV1,
