@@ -47,6 +47,11 @@ typedef enum {
 
 typedef void (*GstAdderFunction) (gpointer out, gpointer in, guint size);
 
+/**
+ * GstAdder:
+ *
+ * The adder object structure.
+ */
 struct _GstAdder {
   GstElement      element;
 
@@ -80,7 +85,7 @@ struct _GstAdder {
   GstPadEventFunction  collect_event;
   GstSegment      segment;
   gboolean        segment_pending;
-  guint64	  segment_position;
+  guint64         segment_position;
 };
 
 struct _GstAdderClass {
@@ -88,7 +93,6 @@ struct _GstAdderClass {
 };
 
 GType    gst_adder_get_type (void);
-gboolean gst_adder_factory_init (GstElementFactory *factory);
 
 G_END_DECLS
 
