@@ -23,6 +23,7 @@
 
 #include <string.h>
 #include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "gstplay-marshal.h"
 
@@ -73,6 +74,8 @@ struct _GstDecodeBin
   gboolean shutting_down;       /* stop pluggin if we're shutting down */
 
   GType queue_type;             /* store the GType of queues, to aid in recognising them */
+
+  GMutex *cb_mutex;             /* Mutex for multi-threaded callbacks, such as removing the fakesink */
 };
 
 struct _GstDecodeBinClass
@@ -109,21 +112,29 @@ typedef struct
  * at runtime */
 typedef struct
 {
-  gint np_sig_id;               /* signal id of new_pad */
-  gint unlink_sig_id;           /* signal id of unlinked */
-  gint nmp_sig_id;              /* signal id of no_more_pads */
-  GstElement *element;          /* the element sending the signal */
   GstDecodeBin *decode_bin;     /* pointer to ourself */
+
+  GstElement *element;          /* the element sending the signal */
+  gint np_sig_id;               /* signal id of new_pad */
+  gint nmp_sig_id;              /* signal id of no_more_pads */
+
+  GstPad *pad;                  /* the pad sending the signal */
+  gint caps_sig_id;             /* signal id of caps */
 }
 GstDynamic;
 
 static void gst_decode_bin_class_init (GstDecodeBinClass * klass);
 static void gst_decode_bin_init (GstDecodeBin * decode_bin);
 static void gst_decode_bin_dispose (GObject * object);
+static void gst_decode_bin_finalize (GObject * object);
 
 static GstStateChangeReturn gst_decode_bin_change_state (GstElement * element,
     GstStateChange transition);
 
+static gboolean add_fakesink (GstDecodeBin * decode_bin);
+static void remove_fakesink (GstDecodeBin * decode_bin);
+
+static void dynamic_free (GstDynamic * dyn);
 static void free_dynamics (GstDecodeBin * decode_bin);
 static void type_found (GstElement * typefind, guint probability,
     GstCaps * caps, GstDecodeBin * decode_bin);
@@ -136,13 +147,15 @@ static void unlinked (GstPad * pad, GstPad * peerpad,
     GstDecodeBin * decode_bin);
 static void new_pad (GstElement * element, GstPad * pad, GstDynamic * dynamic);
 static void no_more_pads (GstElement * element, GstDynamic * dynamic);
+static void new_caps (GstPad * pad, GParamSpec * unused, GstDynamic * dynamic);
 
 static void queue_filled_cb (GstElement * queue, GstDecodeBin * decode_bin);
+static void queue_underrun_cb (GstElement * queue, GstDecodeBin * decode_bin);
 
 static GstElementClass *parent_class;
 static guint gst_decode_bin_signals[LAST_SIGNAL] = { 0 };
 
-static GstElementDetails gst_decode_bin_details =
+static const GstElementDetails gst_decode_bin_details =
 GST_ELEMENT_DETAILS ("Decoder Bin",
     "Generic/Bin/Decoder",
     "Autoplug and decode to raw media",
@@ -207,6 +220,7 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
       GST_TYPE_PAD, GST_TYPE_CAPS);
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_decode_bin_dispose);
+  gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_decode_bin_finalize);
 
   gst_element_class_add_pad_template (gstelement_klass,
       gst_static_pad_template_get (&decoder_bin_sink_template));
@@ -246,7 +260,8 @@ gst_decode_bin_factory_filter (GstPluginFeature * feature,
   klass = gst_element_factory_get_klass (GST_ELEMENT_FACTORY (feature));
   /* only demuxers, decoders and parsers can play */
   if (strstr (klass, "Demux") == NULL &&
-      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL) {
+      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL &&
+      strstr (klass, "Depayloader") == NULL) {
     return FALSE;
   }
 
@@ -292,6 +307,8 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
 {
   GList *factories;
 
+  decode_bin->cb_mutex = g_mutex_new ();
+
   /* first filter out the interesting element factories */
   factories = gst_default_registry_feature_filter (
       (GstPluginFeatureFilter) gst_decode_bin_factory_filter,
@@ -307,7 +324,7 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
   if (!decode_bin->typefind) {
     g_warning ("can't find typefind element, decodebin will not work");
   } else {
-    GstPad *pad;
+    GstPad *pad, *gpad;
 
     /* add the typefind element */
     if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->typefind)) {
@@ -320,8 +337,9 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
     pad = gst_element_get_pad (decode_bin->typefind, "sink");
 
     /* ghost the sink pad to ourself */
-    gst_element_add_pad (GST_ELEMENT (decode_bin),
-        gst_ghost_pad_new ("sink", pad));
+    gpad = gst_ghost_pad_new ("sink", pad);
+    gst_pad_set_active (gpad, TRUE);
+    gst_element_add_pad (GST_ELEMENT (decode_bin), gpad);
 
     gst_object_unref (pad);
 
@@ -331,25 +349,13 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
         g_signal_connect (G_OBJECT (decode_bin->typefind), "have_type",
         G_CALLBACK (type_found), decode_bin);
   }
-  decode_bin->fakesink = gst_element_factory_make ("fakesink", "fakesink");
-  if (!decode_bin->fakesink) {
-    g_warning ("can't find fakesink element, decodebin will not work");
-  } else {
-    GST_OBJECT_FLAG_UNSET (decode_bin->fakesink, GST_ELEMENT_IS_SINK);
-    if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->fakesink)) {
-      g_warning ("Could not add fakesink element, decodebin will not work");
-      gst_object_unref (decode_bin->fakesink);
-      decode_bin->fakesink = NULL;
-    }
-  }
+  add_fakesink (decode_bin);
 
   decode_bin->dynamics = NULL;
   decode_bin->queues = NULL;
   decode_bin->probes = NULL;
 }
 
-static void dynamic_free (GstDynamic * dyn);
-
 static void
 gst_decode_bin_dispose (GObject * object)
 {
@@ -369,26 +375,82 @@ gst_decode_bin_dispose (GObject * object)
   free_dynamics (decode_bin);
 }
 
-static GstDynamic *
-dynamic_create (GstElement * element, GstDecodeBin * decode_bin)
+static void
+gst_decode_bin_finalize (GObject * object)
+{
+  GstDecodeBin *decode_bin = GST_DECODE_BIN (object);
+
+  g_mutex_free (decode_bin->cb_mutex);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+struct DynFind
+{
+  GstElement *elem;
+  GstPad *pad;
+};
+
+static gint
+find_dynamic (GstDynamic * dyn, struct DynFind *info)
+{
+  if (dyn->element == info->elem && dyn->pad == info->pad)
+    return 0;
+  return 1;
+}
+
+/* Add either an element (for dynamic pads/pad-added watching) or a
+ * pad (for delayed caps/notify::caps watching) to the dynamic list,
+ * taking care to ignore repeat entries so we don't end up handling a
+ * pad twice, for example */
+static void
+dynamic_add (GstElement * element, GstPad * pad, GstDecodeBin * decode_bin)
 {
   GstDynamic *dyn;
+  struct DynFind find_info;
+  GList *found;
 
-  GST_DEBUG_OBJECT (element, "dynamic create");
+  g_return_if_fail (element != NULL);
+
+  /* do a search that this entry doesn't already exist */
+  find_info.elem = element;
+  find_info.pad = pad;
+  found = g_list_find_custom (decode_bin->dynamics, &find_info,
+      (GCompareFunc) find_dynamic);
+  if (found != NULL)
+    goto exit;
 
   /* take refs */
-  gst_object_ref (element);
-  gst_object_ref (decode_bin);
-
   dyn = g_new0 (GstDynamic, 1);
-  dyn->element = element;
-  dyn->decode_bin = decode_bin;
-  dyn->np_sig_id = g_signal_connect (G_OBJECT (element), "pad-added",
-      G_CALLBACK (new_pad), dyn);
-  dyn->nmp_sig_id = g_signal_connect (G_OBJECT (element), "no-more-pads",
-      G_CALLBACK (no_more_pads), dyn);
+  dyn->element = gst_object_ref (element);
+  dyn->decode_bin = gst_object_ref (decode_bin);
+  if (pad) {
+    dyn->pad = gst_object_ref (pad);
+    GST_DEBUG_OBJECT (decode_bin, "dynamic create for pad %" GST_PTR_FORMAT,
+        pad);
+    dyn->caps_sig_id = g_signal_connect (G_OBJECT (pad), "notify::caps",
+        G_CALLBACK (new_caps), dyn);
+  } else {
+    GST_DEBUG_OBJECT (decode_bin, "dynamic create for element %"
+        GST_PTR_FORMAT, element);
+    dyn->np_sig_id = g_signal_connect (G_OBJECT (element), "pad-added",
+        G_CALLBACK (new_pad), dyn);
+    dyn->nmp_sig_id = g_signal_connect (G_OBJECT (element), "no-more-pads",
+        G_CALLBACK (no_more_pads), dyn);
+  }
 
-  return dyn;
+  /* and add this element to the dynamic elements */
+  decode_bin->dynamics = g_list_prepend (decode_bin->dynamics, dyn);
+
+  return;
+exit:
+  if (element) {
+    GST_DEBUG_OBJECT (decode_bin, "Dynamic element already added: %"
+        GST_PTR_FORMAT, element);
+  } else {
+    GST_DEBUG_OBJECT (decode_bin, "Dynamic pad already added: %"
+        GST_PTR_FORMAT, pad);
+  }
 }
 
 static void
@@ -397,13 +459,23 @@ dynamic_free (GstDynamic * dyn)
   GST_DEBUG_OBJECT (dyn->decode_bin, "dynamic free");
 
   /* disconnect signals */
-  g_signal_handler_disconnect (G_OBJECT (dyn->element), dyn->np_sig_id);
-  g_signal_handler_disconnect (G_OBJECT (dyn->element), dyn->nmp_sig_id);
+  if (dyn->np_sig_id)
+    g_signal_handler_disconnect (G_OBJECT (dyn->element), dyn->np_sig_id);
+  if (dyn->nmp_sig_id)
+    g_signal_handler_disconnect (G_OBJECT (dyn->element), dyn->nmp_sig_id);
+  if (dyn->caps_sig_id)
+    g_signal_handler_disconnect (G_OBJECT (dyn->pad), dyn->caps_sig_id);
 
-  gst_object_unref (dyn->element);
-  gst_object_unref (dyn->decode_bin);
+  if (dyn->pad)
+    gst_object_unref (dyn->pad);
+  dyn->pad = NULL;
+  if (dyn->element)
+    gst_object_unref (dyn->element);
   dyn->element = NULL;
+
+  gst_object_unref (dyn->decode_bin);
   dyn->decode_bin = NULL;
+
   g_free (dyn);
 }
 
@@ -445,10 +517,14 @@ find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
       /* we only care about the sink templates */
       if (templ->direction == GST_PAD_SINK) {
         GstCaps *intersect;
+        GstCaps *tmpl_caps;
 
         /* try to intersect the caps with the caps of the template */
-        intersect = gst_caps_intersect (caps,
-            gst_static_caps_get (&templ->static_caps));
+        tmpl_caps = gst_static_caps_get (&templ->static_caps);
+
+        intersect = gst_caps_intersect (caps, tmpl_caps);
+        gst_caps_unref (tmpl_caps);
+
         /* check if the intersection is empty */
         if (!gst_caps_is_empty (intersect)) {
           /* non empty intersection, we can use this element */
@@ -490,20 +566,61 @@ free_pad_probes (GstDecodeBin * decode_bin)
   decode_bin->probes = NULL;
 }
 
+static gboolean
+add_fakesink (GstDecodeBin * decode_bin)
+{
+  if (decode_bin->fakesink != NULL)
+    return TRUE;
+
+  g_mutex_lock (decode_bin->cb_mutex);
+
+  decode_bin->fakesink = gst_element_factory_make ("fakesink", "fakesink");
+  if (!decode_bin->fakesink)
+    goto no_fakesink;
+
+  /* hacky, remove sink flag, we don't want our decodebin to become a sink
+   * just because we add a fakesink element to make us ASYNC */
+  GST_OBJECT_FLAG_UNSET (decode_bin->fakesink, GST_ELEMENT_IS_SINK);
+  /* takes ownership */
+  if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->fakesink)) {
+    g_warning ("Could not add fakesink element, decodebin will not work");
+    gst_object_unref (decode_bin->fakesink);
+    decode_bin->fakesink = NULL;
+  }
+  g_mutex_unlock (decode_bin->cb_mutex);
+  return TRUE;
+
+  /* ERRORS */
+no_fakesink:
+  {
+    g_warning ("can't find fakesink element, decodebin will not work");
+    g_mutex_unlock (decode_bin->cb_mutex);
+    return FALSE;
+  }
+}
+
 static void
 remove_fakesink (GstDecodeBin * decode_bin)
 {
+  gboolean removed_fakesink = FALSE;
+
+  if (decode_bin->fakesink == NULL)
+    return;
+
+  g_mutex_lock (decode_bin->cb_mutex);
   if (decode_bin->fakesink) {
-    gst_object_ref (decode_bin->fakesink);
-    gst_bin_remove (GST_BIN (decode_bin), decode_bin->fakesink);
+    GST_DEBUG_OBJECT (decode_bin, "Removing fakesink and marking state dirty");
 
+    /* setting the state to NULL is never async */
     gst_element_set_state (decode_bin->fakesink, GST_STATE_NULL);
-    gst_element_get_state (decode_bin->fakesink, NULL, NULL,
-        GST_CLOCK_TIME_NONE);
-
-    gst_object_unref (decode_bin->fakesink);
+    gst_bin_remove (GST_BIN (decode_bin), decode_bin->fakesink);
     decode_bin->fakesink = NULL;
 
+    removed_fakesink = TRUE;
+  }
+  g_mutex_unlock (decode_bin->cb_mutex);
+
+  if (removed_fakesink) {
     free_pad_probes (decode_bin);
 
     gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
@@ -511,6 +628,7 @@ remove_fakesink (GstDecodeBin * decode_bin)
   }
 }
 
+/* this should be implemented with _pad_block() */
 static gboolean
 pad_probe (GstPad * pad, GstMiniObject * data, GstDecodeBin * decode_bin)
 {
@@ -529,12 +647,18 @@ pad_probe (GstPad * pad, GstMiniObject * data, GstDecodeBin * decode_bin)
           ((GST_EVENT_TYPE (data) == GST_EVENT_EOS) ||
               (GST_EVENT_TYPE (data) == GST_EVENT_TAG) ||
               (GST_EVENT_TYPE (data) == GST_EVENT_FLUSH_START))) {
+        /* FIXME, what about NEWSEGMENT? really, use _pad_block()... */
         if (!pdata->done)
           decode_bin->numwaiting--;
         pdata->done = TRUE;
       }
-    } else if (!(pdata->done))
+    }
+
+    if (!(pdata->done)) {
+      GST_LOG_OBJECT (decode_bin, "Pad probe on pad %" GST_PTR_FORMAT
+          " but pad %" GST_PTR_FORMAT " still needs data.", pad, pdata->pad);
       alldone = FALSE;
+    }
   }
   if (alldone)
     remove_fakesink (decode_bin);
@@ -586,7 +710,8 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
   mimetype = gst_structure_get_name (structure);
 
   /* first see if this is raw. If the type is raw, we can
-   * create a ghostpad for this pad. */
+   * create a ghostpad for this pad. It's possible that the caps are not
+   * fixed. */
   if (mimetype_is_raw (mimetype)) {
     gchar *padname;
     GstPad *ghost;
@@ -598,12 +723,14 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
 
     /* make it a ghostpad */
     ghost = gst_ghost_pad_new (padname, pad);
+    gst_pad_set_active (ghost, TRUE);
     gst_element_add_pad (GST_ELEMENT (decode_bin), ghost);
 
     data = g_new0 (PadProbeData, 1);
     data->pad = pad;
     data->done = FALSE;
 
+    /* FIXME, use _pad_block() */
     data->sigid = gst_pad_add_data_probe (pad, G_CALLBACK (pad_probe),
         decode_bin);
     decode_bin->numwaiting++;
@@ -632,27 +759,42 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
       /* no compatible elements, we cannot go on */
       goto unknown_type;
 
-    try_to_link_1 (decode_bin, element, pad, to_try);
+    if (try_to_link_1 (decode_bin, element, pad, to_try) == NULL) {
+      g_list_free (to_try);
+      GST_LOG_OBJECT (pad, "none of the allegedly available elements usable");
+      goto unknown_type;
+    }
+
     /* can free the list again now */
     g_list_free (to_try);
   }
   return;
 
+  /* ERRORS */
 unknown_type:
   {
-    GST_LOG_OBJECT (pad, "unkown type found, fire signal");
+    GST_LOG_OBJECT (pad, "unknown type found, fire signal");
     g_signal_emit (G_OBJECT (decode_bin),
         gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
+
+    gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
+        gst_missing_decoder_message_new (GST_ELEMENT_CAST (decode_bin), caps));
     return;
   }
 dont_know_yet:
   {
-    GST_LOG_OBJECT (pad, "type is not known yet, waiting to close link");
-    return;
+    GST_LOG_OBJECT (pad, "type is not known yet");
+    goto setup_caps_delay;
   }
 many_types:
   {
-    GST_LOG_OBJECT (pad, "many possible types, waiting to close link");
+    GST_LOG_OBJECT (pad, "many possible types");
+    goto setup_caps_delay;
+  }
+setup_caps_delay:
+  {
+    GST_LOG_OBJECT (pad, "setting up a delayed link");
+    dynamic_add (element, pad, decode_bin);
     return;
   }
 }
@@ -664,7 +806,7 @@ is_demuxer_element (GstElement * srcelement)
 {
   GstElementFactory *srcfactory;
   GstElementClass *elemclass;
-  GList *templates, *walk;
+  GList *walk;
   const gchar *klass;
   gint potential_src_pads = 0;
 
@@ -672,14 +814,14 @@ is_demuxer_element (GstElement * srcelement)
   klass = gst_element_factory_get_klass (srcfactory);
 
   /* Can't be a demuxer unless it has Demux in the klass name */
-  if (!strstr (klass, "Demux"))
+  if (klass == NULL || !strstr (klass, "Demux"))
     return FALSE;
 
   /* Walk the src pad templates and count how many the element
    * might produce */
   elemclass = GST_ELEMENT_GET_CLASS (srcelement);
 
-  walk = templates = gst_element_class_get_pad_template_list (elemclass);
+  walk = gst_element_class_get_pad_template_list (elemclass);
   while (walk != NULL) {
     GstPadTemplate *templ;
 
@@ -782,7 +924,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     GST_DEBUG_OBJECT (decode_bin, "adding %s", GST_OBJECT_NAME (element));
     gst_bin_add (GST_BIN (decode_bin), element);
 
-    /* set to ready first so it is ready */
+    /* set to READY first so it is ready, duh. */
     gst_element_set_state (element, GST_STATE_READY);
 
     if ((ret = gst_pad_link (usedsrcpad, sinkpad)) != GST_PAD_LINK_OK) {
@@ -794,17 +936,19 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
        * other elements, the element will be disposed. */
       gst_element_set_state (element, GST_STATE_NULL);
       gst_bin_remove (GST_BIN (decode_bin), element);
-
     } else {
       guint sig;
 
       GST_DEBUG_OBJECT (decode_bin, "linked on pad %s:%s",
           GST_DEBUG_PAD_NAME (usedsrcpad));
 
+      /* configure the queue some more */
       if (queue != NULL) {
         decode_bin->queues = g_list_append (decode_bin->queues, queue);
         g_signal_connect (G_OBJECT (queue),
             "overrun", G_CALLBACK (queue_filled_cb), decode_bin);
+        g_signal_connect (G_OBJECT (queue),
+            "underrun", G_CALLBACK (queue_underrun_cb), decode_bin);
       }
 
       /* The link worked, now figure out what it was that we connected */
@@ -861,31 +1005,31 @@ get_our_ghost_pad (GstDecodeBin * decode_bin, GstPad * pad)
     return NULL;
   }
 
-  pad_it = gst_element_iterate_pads (GST_ELEMENT (decode_bin));
+  /* our ghostpads are the sourcepads */
+  pad_it = gst_element_iterate_src_pads (GST_ELEMENT (decode_bin));
   while (!done) {
+    db_pad = NULL;
     switch (gst_iterator_next (pad_it, (gpointer) & db_pad)) {
       case GST_ITERATOR_OK:
         GST_DEBUG_OBJECT (decode_bin, "looking at pad %s:%s",
             GST_DEBUG_PAD_NAME (db_pad));
-        if (GST_IS_GHOST_PAD (db_pad) && GST_PAD_IS_SRC (db_pad)) {
+        if (GST_IS_GHOST_PAD (db_pad)) {
           GstPad *target_pad = NULL;
 
           target_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (db_pad));
+          done = (target_pad == pad);
+          if (target_pad)
+            gst_object_unref (target_pad);
 
-          if (target_pad == pad) {      /* Found our ghost pad */
+          if (done) {
+            /* Found our ghost pad */
             GST_DEBUG_OBJECT (decode_bin, "found ghostpad %s:%s for pad %s:%s",
                 GST_DEBUG_PAD_NAME (db_pad), GST_DEBUG_PAD_NAME (pad));
-            done = TRUE;
             break;
-          } else {              /* Not the right one */
-            gst_object_unref (db_pad);
-            db_pad = NULL;
           }
-        } else {
-          gst_object_unref (db_pad);
-          db_pad = NULL;
         }
-
+        /* Not the right one */
+        gst_object_unref (db_pad);
         break;
       case GST_ITERATOR_RESYNC:
         gst_iterator_resync (pad_it);
@@ -963,10 +1107,10 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
       GstObject *parent = gst_pad_get_parent (peer);
 
       if (parent) {
-        GstElement *grandparent = GST_ELEMENT (gst_object_get_parent (parent));
+        GstObject *grandparent = gst_object_get_parent (parent);
 
-        if (grandparent) {
-          if (grandparent != GST_ELEMENT (decode_bin)) {
+        if (grandparent != NULL) {
+          if (GST_ELEMENT (grandparent) != GST_ELEMENT (decode_bin)) {
             GST_DEBUG_OBJECT (decode_bin, "dead end pad %s:%s parent %s",
                 GST_DEBUG_PAD_NAME (peer), GST_OBJECT_NAME (grandparent));
           } else {
@@ -990,6 +1134,48 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
   gst_bin_remove (GST_BIN (decode_bin), elem);
 }
 
+/* there are @bytes bytes in @queue, enlarge it 
+ *
+ * Returns: new max number of bytes in @queue
+ */
+static guint
+queue_enlarge (GstElement * queue, guint bytes, GstDecodeBin * decode_bin)
+{
+  /* Increase the queue size by 1Mbyte if it is over 1Mb, else double its current limit
+   */
+  if (bytes > 1024 * 1024)
+    bytes += 1024 * 1024;
+  else
+    bytes *= 2;
+
+  GST_DEBUG_OBJECT (decode_bin,
+      "increasing queue %s max-size-bytes to %d", GST_ELEMENT_NAME (queue),
+      bytes);
+  g_object_set (G_OBJECT (queue), "max-size-bytes", bytes, NULL);
+
+  return bytes;
+}
+
+/* this callback is called when our queues fills up or are empty
+ * We then check the status of all other queues to make sure we
+ * never have an empty and full queue at the same time since that
+ * would block dataflow. In the case of a filled queue, we make
+ * it larger.
+ */
+static void
+queue_underrun_cb (GstElement * queue, GstDecodeBin * decode_bin)
+{
+  /* FIXME: we don't really do anything here for now. Ideally we should
+   * see if some of the queues are filled and increase their values
+   * in that case. 
+   * Note: be very carefull with thread safety here as this underrun
+   * signal is done from the streaming thread of queue srcpad which
+   * is different from the pad_added (where we add the queue to the
+   * list) and the overrun signals that are signalled from the
+   * demuxer thread.
+   */
+}
+
 /* Make sure we don't have a full queue and empty queue situation */
 static void
 queue_filled_cb (GstElement * queue, GstDecodeBin * decode_bin)
@@ -998,43 +1184,48 @@ queue_filled_cb (GstElement * queue, GstDecodeBin * decode_bin)
   gboolean increase = FALSE;
   guint bytes;
 
+  /* get current byte level from the queue that is filled */
   g_object_get (G_OBJECT (queue), "current-level-bytes", &bytes, NULL);
   GST_DEBUG_OBJECT (decode_bin, "One of the queues is full at %d bytes", bytes);
 
-  if (bytes > (20 * 1024 * 1024)) {
-    GST_WARNING_OBJECT (decode_bin,
-        "Queue is bigger than 20Mbytes, something else is going wrong");
-    return;
-  }
+  /* we do not buffer more than 20Mb */
+  if (bytes > (20 * 1024 * 1024))
+    goto too_large;
 
+  /* check all other queue to see if one is empty, in that case
+   * we need to enlarge @queue */
   for (tmp = decode_bin->queues; tmp; tmp = g_list_next (tmp)) {
     GstElement *aqueue = GST_ELEMENT (tmp->data);
-    guint levelbytes = -1;
+    guint levelbytes = 0;
 
     if (aqueue != queue) {
-      g_object_get (G_OBJECT (aqueue),
-          "current-level-bytes", &levelbytes, NULL);
+      g_object_get (G_OBJECT (aqueue), "current-level-bytes", &levelbytes,
+          NULL);
       if (levelbytes == 0) {
+        /* yup, found an empty queue, we can stop the search and
+         * need to enlarge the queue */
         increase = TRUE;
+        break;
       }
     }
   }
 
   if (increase) {
-    /* 
-     * Increase the queue size by 1Mbyte if it is over 1Mb, else double its current limit
-     */
-    if (bytes > 1024 * 1024)
-      bytes += 1024 * 1024;
-    else
-      bytes *= 2;
-    GST_DEBUG_OBJECT (decode_bin,
-        "One of the other queues is empty, increasing queue byte limit to %d",
-        bytes);
-    g_object_set (G_OBJECT (queue), "max-size-bytes", bytes, NULL);
-  } else
+    /* enlarge @queue */
+    queue_enlarge (queue, bytes, decode_bin);
+  } else {
     GST_DEBUG_OBJECT (decode_bin,
         "Queue is full but other queues are not empty, not doing anything");
+  }
+  return;
+
+  /* errors */
+too_large:
+  {
+    GST_WARNING_OBJECT (decode_bin,
+        "Queue is bigger than 20Mbytes, something else is going wrong");
+    return;
+  }
 }
 
 /* This function will be called when a dynamic pad is created on an element.
@@ -1067,28 +1258,23 @@ new_pad (GstElement * element, GstPad * pad, GstDynamic * dynamic)
   return;
 
 shutting_down1:
-  GST_OBJECT_UNLOCK (decode_bin);
-  return;
-
+  {
+    GST_OBJECT_UNLOCK (decode_bin);
+    return;
+  }
 shutting_down2:
-  GST_STATE_UNLOCK (decode_bin);
-  return;
+  {
+    GST_STATE_UNLOCK (decode_bin);
+    return;
+  }
 }
 
-/* this signal is fired when an element signals the no_more_pads signal.
- * This means that the element will not generate more dynamic pads and
- * we can remove the element from the list of dynamic elements. When we
- * have no more dynamic elements in the pipeline, we can fire a no_more_pads
- * signal ourselves. */
 static void
-no_more_pads (GstElement * element, GstDynamic * dynamic)
+dynamic_remove (GstDynamic * dynamic)
 {
   GstDecodeBin *decode_bin = dynamic->decode_bin;
 
-  GST_DEBUG_OBJECT (decode_bin, "no more pads on element %s",
-      GST_ELEMENT_NAME (element));
-
-  /* remove the element from the list of dynamic elements */
+  /* remove the dynamic from the list of dynamics */
   decode_bin->dynamics = g_list_remove (decode_bin->dynamics, dynamic);
   dynamic_free (dynamic);
 
@@ -1108,6 +1294,33 @@ no_more_pads (GstElement * element, GstDynamic * dynamic)
   }
 }
 
+/* this signal is fired when an element signals the no_more_pads signal.
+ * This means that the element will not generate more dynamic pads and
+ * we can remove the element from the list of dynamic elements. When we
+ * have no more dynamic elements in the pipeline, we can fire a no_more_pads
+ * signal ourselves. */
+static void
+no_more_pads (GstElement * element, GstDynamic * dynamic)
+{
+  GST_DEBUG_OBJECT (dynamic->decode_bin, "no more pads on element %s",
+      GST_ELEMENT_NAME (element));
+
+  dynamic_remove (dynamic);
+}
+
+static void
+new_caps (GstPad * pad, GParamSpec * unused, GstDynamic * dynamic)
+{
+  GST_DEBUG_OBJECT (dynamic->decode_bin, "delayed link triggered");
+
+  new_pad (dynamic->element, pad, dynamic);
+
+  /* assume it worked and remove the dynamic */
+  dynamic_remove (dynamic);
+
+  return;
+}
+
 static gboolean
 is_our_kid (GstElement * e, GstDecodeBin * decode_bin)
 {
@@ -1123,17 +1336,61 @@ is_our_kid (GstElement * e, GstDecodeBin * decode_bin)
   return ret;
 }
 
+static gboolean
+elem_is_dynamic (GstElement * element, GstDecodeBin * decode_bin)
+{
+  GList *pads;
+
+  /* loop over all the padtemplates */
+  for (pads = GST_ELEMENT_GET_CLASS (element)->padtemplates; pads;
+      pads = g_list_next (pads)) {
+    GstPadTemplate *templ = GST_PAD_TEMPLATE (pads->data);
+    const gchar *templ_name;
+
+    /* we are only interested in source pads */
+    if (GST_PAD_TEMPLATE_DIRECTION (templ) != GST_PAD_SRC)
+      continue;
+
+    templ_name = GST_PAD_TEMPLATE_NAME_TEMPLATE (templ);
+    GST_DEBUG_OBJECT (decode_bin, "got a source pad template %s", templ_name);
+
+    /* figure out what kind of pad this is */
+    switch (GST_PAD_TEMPLATE_PRESENCE (templ)) {
+      case GST_PAD_SOMETIMES:
+      {
+        /* try to get the pad to see if it is already created or
+         * not */
+        GstPad *pad = gst_element_get_pad (element, templ_name);
+
+        if (pad) {
+          GST_DEBUG_OBJECT (decode_bin, "got the pad for sometimes template %s",
+              templ_name);
+          gst_object_unref (pad);
+        } else {
+          GST_DEBUG_OBJECT (decode_bin,
+              "did not get the sometimes pad of template %s", templ_name);
+          /* we have an element that will create dynamic pads */
+          return TRUE;
+        }
+        break;
+      }
+      default:
+        /* Don't care about ALWAYS or REQUEST pads */
+        break;
+    }
+  }
+  return FALSE;
+}
+
 /* This function will be called when a pad is disconnected for some reason */
 static void
 unlinked (GstPad * pad, GstPad * peerpad, GstDecodeBin * decode_bin)
 {
-  GstDynamic *dyn;
   GstElement *element, *peer;
 
   /* inactivate pad */
   gst_pad_set_active (pad, GST_ACTIVATE_NONE);
 
-  element = gst_pad_get_parent_element (pad);
   peer = gst_pad_get_parent_element (peerpad);
 
   if (!is_our_kid (peer, decode_bin))
@@ -1145,18 +1402,16 @@ unlinked (GstPad * pad, GstPad * peerpad, GstDecodeBin * decode_bin)
   /* remove all elements linked to the peerpad */
   remove_element_chain (decode_bin, peerpad);
 
-  /* if an element removes two pads, then we don't want this twice */
-  /* FIXME: decode_bin->dynamics doesn't contain a list of GstElements, it
-   * has GstDynamic structures */
-  if (g_list_find (decode_bin->dynamics, element) != NULL)
-    goto exit;
+  /* Re-add as a dynamic element if needed, via close_link */
+  element = gst_pad_get_parent_element (pad);
+  if (element) {
+    if (elem_is_dynamic (element, decode_bin))
+      dynamic_add (element, NULL, decode_bin);
 
-  dyn = dynamic_create (element, decode_bin);
-  /* and add this element to the dynamic elements */
-  decode_bin->dynamics = g_list_prepend (decode_bin->dynamics, dyn);
+    gst_object_unref (element);
+  }
 
 exit:
-  gst_object_unref (element);
   gst_object_unref (peer);
 }
 
@@ -1234,15 +1489,11 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
     }
   }
   if (dynamic) {
-    GstDynamic *dyn;
-
     GST_DEBUG_OBJECT (decode_bin, "got a dynamic element here");
     /* ok, this element has dynamic pads, set up the signal handlers to be
      * notified of them */
 
-    dyn = dynamic_create (element, decode_bin);
-    /* and add this element to the dynamic elements */
-    decode_bin->dynamics = g_list_prepend (decode_bin->dynamics, dyn);
+    dynamic_add (element, NULL, decode_bin);
   }
 
   /* Check if this is an element with more than 1 pad. If this element
@@ -1290,6 +1541,18 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
 
   GST_DEBUG_OBJECT (decode_bin, "typefind found caps %" GST_PTR_FORMAT, caps);
 
+  /* special-case text/plain: we only want to accept it as a raw type if it
+   * comes from a subtitel parser element or a demuxer, but not if it is the
+   * type of the entire stream, in which case we just want to error out */
+  if (typefind == decode_bin->typefind &&
+      gst_structure_has_name (gst_caps_get_structure (caps, 0), "text/plain")) {
+    gst_element_no_more_pads (GST_ELEMENT (decode_bin));
+    /* we can't handle this type of stream */
+    GST_ELEMENT_ERROR (decode_bin, STREAM, WRONG_TYPE, (NULL),
+        ("decodebin cannot decode plain text files"));
+    goto shutting_down;
+  }
+
   /* autoplug the new pad with the caps that the signal gave us. */
   pad = gst_element_get_pad (typefind, "src");
   close_pad_link (typefind, pad, caps, decode_bin, FALSE);
@@ -1324,7 +1587,8 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
 
   typefind_pad = gst_element_get_pad (decode_bin->typefind, "src");
   if (GST_IS_PAD (typefind_pad)) {
-    g_signal_handlers_block_by_func (typefind_pad, unlinked, decode_bin);
+    g_signal_handlers_block_by_func (typefind_pad, (gpointer) unlinked,
+        decode_bin);
   }
 
   elem_it = gst_bin_iterate_elements (GST_BIN (decode_bin));
@@ -1335,7 +1599,9 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
       case GST_ITERATOR_OK:
         if (element != decode_bin->typefind && element != decode_bin->fakesink) {
           GST_DEBUG_OBJECT (element, "removing autoplugged element");
-          g_signal_handlers_disconnect_by_func (element, unlinked, decode_bin);
+          g_signal_handlers_disconnect_by_func (element, (gpointer) unlinked,
+              decode_bin);
+          gst_element_set_state (element, GST_STATE_NULL);
           gst_bin_remove (GST_BIN (decode_bin), element);
         }
         gst_object_unref (element);
@@ -1353,6 +1619,7 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
   }
   gst_iterator_free (elem_it);
 
+  done = FALSE;
   gpad_it = gst_element_iterate_pads (GST_ELEMENT (decode_bin));
   while (!done) {
     GstPad *pad = NULL;
@@ -1381,7 +1648,8 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
   gst_iterator_free (gpad_it);
 
   if (GST_IS_PAD (typefind_pad)) {
-    g_signal_handlers_unblock_by_func (typefind_pad, unlinked, decode_bin);
+    g_signal_handlers_unblock_by_func (typefind_pad, (gpointer) unlinked,
+        decode_bin);
     gst_object_unref (typefind_pad);
   }
 }
@@ -1399,11 +1667,16 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
       decode_bin->numpads = 0;
       decode_bin->numwaiting = 0;
       decode_bin->dynamics = NULL;
+      if (decode_bin->typefind == NULL)
+        goto missing_typefind;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_OBJECT_LOCK (decode_bin);
       decode_bin->shutting_down = FALSE;
       GST_OBJECT_UNLOCK (decode_bin);
+
+      if (!add_fakesink (decode_bin))
+        goto missing_fakesink;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -1418,11 +1691,13 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret == GST_STATE_CHANGE_FAILURE)
+    return ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
       break;
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
     case GST_STATE_CHANGE_READY_TO_NULL:
       free_dynamics (decode_bin);
       free_pad_probes (decode_bin);
@@ -1433,6 +1708,21 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   }
 
   return ret;
+/* ERRORS */
+missing_typefind:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "typefind"));
+    GST_ELEMENT_ERROR (element, CORE, MISSING_PLUGIN, (NULL), ("no typefind!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
+missing_fakesink:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "fakesink"));
+    GST_ELEMENT_ERROR (element, CORE, MISSING_PLUGIN, (NULL), ("no fakesink!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
 }
 
 static gboolean
