@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
- * Copyright (c) 2004 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
+ * Copyright (C) 2004 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
+ * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,9 +31,9 @@
 #include "gstsubparse.h"
 #include "gstssaparse.h"
 #include "samiparse.h"
+#include "tmplayerparse.h"
 
-GST_DEBUG_CATEGORY_STATIC (sub_parse_debug);
-#define GST_CAT_DEFAULT sub_parse_debug
+GST_DEBUG_CATEGORY (sub_parse_debug);
 
 #define DEFAULT_ENCODING   NULL
 
@@ -61,7 +62,8 @@ GST_ELEMENT_DETAILS ("Subtitle parser",
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("application/x-subtitle; application/x-subtitle-sami")
+    GST_STATIC_CAPS ("application/x-subtitle; application/x-subtitle-sami; "
+        "application/x-subtitle-tmplayer")
     );
 #else
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -143,6 +145,10 @@ gst_sub_parse_dispose (GObject * object)
     g_free (subparse->encoding);
     subparse->encoding = NULL;
   }
+  if (subparse->textbuf) {
+    g_string_free (subparse->textbuf, TRUE);
+    subparse->textbuf = NULL;
+  }
   sami_context_deinit (&subparse->state);
 
   GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
@@ -347,7 +353,8 @@ convert_encoding (GstSubParse * self, const gchar * str, gsize len)
         NULL, NULL, NULL);
   }
 
-  GST_LOG_OBJECT (self, "successfully converted %d characters from %s to UTF-8"
+  GST_LOG_OBJECT (self,
+      "successfully converted %" G_GSIZE_FORMAT " characters from %s to UTF-8"
       "%s", len, encoding, (err) ? " , using ISO-8859-15 as fallback" : "");
 
   return ret;
@@ -389,10 +396,6 @@ parse_mdvdsub (ParserState * state, const gchar * line)
   guint start_frame, end_frame;
   gint64 clip_start = 0, clip_stop = 0;
   gboolean in_seg = FALSE;
-
-  /* FIXME: hardcoded for now, but detecting the correct value is
-   * not going to be easy, I suspect... */
-  const double frames_per_sec = 24000 / 1001.;
   GString *markup;
   gchar *ret;
 
@@ -407,8 +410,32 @@ parse_mdvdsub (ParserState * state, const gchar * line)
     return NULL;
   }
 
-  state->start_time = (start_frame - 1000) / frames_per_sec * GST_SECOND;
-  state->duration = (end_frame - start_frame) / frames_per_sec * GST_SECOND;
+  /* skip the {%u}{%u} part */
+  line = strchr (line, '}') + 1;
+  line = strchr (line, '}') + 1;
+
+  /* see if there's a first line with a framerate */
+  if (state->fps == 0.0 && start_frame == 1 && end_frame == 1) {
+    gchar *rest, *end = NULL;
+
+    rest = g_strdup (line);
+    g_strdelimit (rest, ",", '.');
+    state->fps = g_ascii_strtod (rest, &end);
+    if (end == rest)
+      state->fps = 0.0;
+    GST_INFO ("framerate from file: %f ('%s')", state->fps, rest);
+    g_free (rest);
+    return NULL;
+  }
+
+  if (state->fps == 0.0) {
+    /* FIXME: hardcoded for now, is there a better way/assumption? */
+    state->fps = 24000.0 / 1001.0;
+    GST_INFO ("no framerate specified, assuming %f", state->fps);
+  }
+
+  state->start_time = start_frame / state->fps * GST_SECOND;
+  state->duration = (end_frame - start_frame) / state->fps * GST_SECOND;
 
   /* Check our segment start/stop */
   in_seg = gst_segment_clip (state->segment, GST_FORMAT_TIME,
@@ -423,10 +450,6 @@ parse_mdvdsub (ParserState * state, const gchar * line)
     return NULL;
   }
 
-  /* skip the {%u}{%u} part */
-  line = strchr (line, '}') + 1;
-  line = strchr (line, '}') + 1;
-
   markup = g_string_new (NULL);
   while (1) {
     italic = FALSE;
@@ -472,6 +495,20 @@ parse_mdvdsub (ParserState * state, const gchar * line)
   return ret;
 }
 
+static void
+strip_trailing_newlines (gchar * txt)
+{
+  if (txt) {
+    guint len;
+
+    len = strlen (txt);
+    while (len > 1 && txt[len - 1] == '\n') {
+      txt[len - 1] = '\0';
+      --len;
+    }
+  }
+}
+
 /* we want to escape text in general, but retain basic markup like
  * <i></i>, <u></u>, and <b></b>. The easiest and safest way is to
  * just unescape a white list of allowed markups again after
@@ -510,6 +547,107 @@ subrip_unescape_formatting (gchar * txt)
   }
 }
 
+
+static gboolean
+subrip_remove_unhandled_tag (gchar * start, gchar * stop)
+{
+  gchar *tag, saved;
+
+  tag = start + strlen ("&lt;");
+  if (*tag == '/')
+    ++tag;
+
+  if (g_ascii_tolower (*tag) < 'a' || g_ascii_tolower (*tag) > 'z')
+    return FALSE;
+
+  saved = *stop;
+  *stop = '\0';
+  GST_LOG ("removing unhandled tag '%s'", start);
+  *stop = saved;
+  g_memmove (start, stop, strlen (stop) + 1);
+  return TRUE;
+}
+
+/* remove tags we haven't explicitly allowed earlier on, like font tags
+ * for example */
+static void
+subrip_remove_unhandled_tags (gchar * txt)
+{
+  gchar *pos, *gt;
+
+  for (pos = txt; pos != NULL && *pos != '\0'; ++pos) {
+    if (strncmp (pos, "&lt;", 4) == 0 && (gt = strstr (pos + 4, "&gt;"))) {
+      if (subrip_remove_unhandled_tag (pos, gt + strlen ("&gt;")))
+        --pos;
+    }
+  }
+}
+
+/* we only allow <i>, <u> and <b>, so let's take a simple approach. This code
+ * assumes the input has been escaped and subrip_unescape_formatting() has then
+ * been run over the input! This function adds missing closing markup tags and
+ * removes broken closing tags for tags that have never been opened. */
+static void
+subrip_fix_up_markup (gchar ** p_txt)
+{
+  gchar *cur, *next_tag;
+  gchar open_tags[32];
+  guint num_open_tags = 0;
+
+  g_assert (*p_txt != NULL);
+
+  cur = *p_txt;
+  while (*cur != '\0') {
+    next_tag = strchr (cur, '<');
+    if (next_tag == NULL)
+      break;
+    ++next_tag;
+    switch (*next_tag) {
+      case '/':{
+        ++next_tag;
+        if (num_open_tags == 0 || open_tags[num_open_tags - 1] != *next_tag) {
+          GST_LOG ("broken input, closing tag '%c' is not open", *next_tag);
+          g_memmove (next_tag - 2, next_tag + 2, strlen (next_tag + 2) + 1);
+          next_tag -= 2;
+        } else {
+          /* it's all good, closing tag which is open */
+          --num_open_tags;
+        }
+        break;
+      }
+      case 'i':
+      case 'b':
+      case 'u':
+        if (num_open_tags == G_N_ELEMENTS (open_tags))
+          return;               /* something dodgy is going on, stop parsing */
+        open_tags[num_open_tags] = *next_tag;
+        ++num_open_tags;
+        break;
+      default:
+        GST_ERROR ("unexpected tag '%c' (%s)", *next_tag, next_tag);
+        g_assert_not_reached ();
+        break;
+    }
+    cur = next_tag;
+  }
+
+  if (num_open_tags > 0) {
+    GString *s;
+
+    s = g_string_new (*p_txt);
+    while (num_open_tags > 0) {
+      GST_LOG ("adding missing closing tag '%c'", open_tags[num_open_tags - 1]);
+      g_string_append_c (s, '<');
+      g_string_append_c (s, '/');
+      g_string_append_c (s, open_tags[num_open_tags - 1]);
+      g_string_append_c (s, '>');
+      --num_open_tags;
+    }
+    g_free (*p_txt);
+    *p_txt = g_string_free (s, FALSE);
+  }
+}
+
 static gchar *
 parse_subrip (ParserState * state, const gchar * line)
 {
@@ -568,6 +706,9 @@ parse_subrip (ParserState * state, const gchar * line)
         g_string_truncate (state->buf, 0);
         state->state = 0;
         subrip_unescape_formatting (ret);
+        subrip_remove_unhandled_tags (ret);
+        strip_trailing_newlines (ret);
+        subrip_fix_up_markup (&ret);
         return ret;
       }
       return NULL;
@@ -668,6 +809,7 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
   static gboolean need_init_regexps = TRUE;
   static regex_t mdvd_rx;
   static regex_t subrip_rx;
+  guint n1, n2, n3;
 
   /* initialize the regexps used the first time around */
   if (need_init_regexps) {
@@ -677,7 +819,7 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     need_init_regexps = FALSE;
     if ((err = regcomp (&mdvd_rx, "^\\{[0-9]+\\}\\{[0-9]+\\}",
                 REG_EXTENDED | REG_NEWLINE | REG_NOSUB) != 0) ||
-        (err = regcomp (&subrip_rx, "^1(\x0d)?\x0a"
+        (err = regcomp (&subrip_rx, "^[1-9]([0-9]){0,3}(\x0d)?\x0a"
                 "[0-9][0-9]:[0-9][0-9]:[0-9][0-9],[0-9]{3}"
                 " --> [0-9][0-9]:[0-9][0-9]:[0-9][0-9],[0-9]{3}",
                 REG_EXTENDED | REG_NEWLINE | REG_NOSUB)) != 0) {
@@ -703,6 +845,15 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     GST_LOG ("SAMI (time based) format detected");
     return GST_SUB_PARSE_FORMAT_SAMI;
   }
+  /* we're boldly assuming the first subtitle appears within the first hour */
+  if (sscanf (match_str, "0:%02u:%02u:", &n1, &n2) == 2 ||
+      sscanf (match_str, "0:%02u:%02u=", &n1, &n2) == 2 ||
+      sscanf (match_str, "00:%02u:%02u:", &n1, &n2) == 2 ||
+      sscanf (match_str, "00:%02u:%02u=", &n1, &n2) == 2 ||
+      sscanf (match_str, "00:%02u:%02u,%u=", &n1, &n2, &n3) == 3) {
+    GST_LOG ("TMPlayer (time based) format detected");
+    return GST_SUB_PARSE_FORMAT_TMPLAYER;
+  }
 
   GST_DEBUG ("no subtitle format detected");
   return GST_SUB_PARSE_FORMAT_UNKNOWN;
@@ -740,6 +891,9 @@ gst_sub_parse_format_autodetect (GstSubParse * self)
       self->parse_line = parse_sami;
       sami_context_init (&self->state);
       return gst_caps_new_simple ("text/x-pango-markup", NULL);
+    case GST_SUB_PARSE_FORMAT_TMPLAYER:
+      self->parse_line = parse_tmplayer;
+      return gst_caps_new_simple ("text/plain", NULL);
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
     default:
       GST_DEBUG ("no subtitle format detected");
@@ -792,26 +946,29 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
     /* Set segment on our parser state machine */
     self->state.segment = self->segment;
     /* Now parse the line, out of segment lines will just return NULL */
-    GST_DEBUG ("Parsing line '%s'", line);
+    GST_LOG_OBJECT (self, "Parsing line '%s'", line);
     subtitle = self->parse_line (&self->state, line);
     g_free (line);
 
     if (subtitle) {
       guint subtitle_len = strlen (subtitle);
 
+      /* +1 for terminating NUL character */
       ret = gst_pad_alloc_buffer_and_set_caps (self->srcpad,
-          GST_BUFFER_OFFSET_NONE, subtitle_len,
+          GST_BUFFER_OFFSET_NONE, subtitle_len + 1,
           GST_PAD_CAPS (self->srcpad), &buf);
 
       if (ret == GST_FLOW_OK) {
-        memcpy (GST_BUFFER_DATA (buf), subtitle, subtitle_len);
+        /* copy terminating NUL character as well */
+        memcpy (GST_BUFFER_DATA (buf), subtitle, subtitle_len + 1);
+        GST_BUFFER_SIZE (buf) = subtitle_len;
         GST_BUFFER_TIMESTAMP (buf) = self->state.start_time;
         GST_BUFFER_DURATION (buf) = self->state.duration;
 
         gst_segment_set_last_stop (self->segment, GST_FORMAT_TIME,
             self->state.start_time);
 
-        GST_DEBUG ("Sending text '%s', %" GST_TIME_FORMAT " + %"
+        GST_DEBUG_OBJECT (self, "Sending text '%s', %" GST_TIME_FORMAT " + %"
             GST_TIME_FORMAT, subtitle, GST_TIME_ARGS (self->state.start_time),
             GST_TIME_ARGS (self->state.duration));
 
@@ -835,8 +992,7 @@ gst_sub_parse_chain (GstPad * sinkpad, GstBuffer * buf)
   GstFlowReturn ret;
   GstSubParse *self;
 
-  GST_DEBUG ("gst_sub_parse_chain");
-  self = GST_SUBPARSE (gst_pad_get_parent (sinkpad));
+  self = GST_SUBPARSE (GST_PAD_PARENT (sinkpad));
 
   /* Push newsegment if needed */
   if (self->need_segment) {
@@ -849,8 +1005,6 @@ gst_sub_parse_chain (GstPad * sinkpad, GstBuffer * buf)
 
   ret = handle_buffer (self, buf);
 
-  gst_object_unref (self);
-
   return ret;
 }
 
@@ -933,9 +1087,11 @@ gst_sub_parse_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       /* format detection will init the parser state */
-      self->offset = self->next_offset = 0;
+      self->offset = 0;
+      self->next_offset = 0;
       self->parser_type = GST_SUB_PARSE_FORMAT_UNKNOWN;
       self->valid_utf8 = TRUE;
+      g_string_truncate (self->textbuf, 0);
       break;
     default:
       break;
@@ -961,11 +1117,16 @@ gst_sub_parse_change_state (GstElement * element, GstStateChange transition)
  * Typefind support.
  */
 
+/* FIXME 0.11: these caps are ugly, use app/x-subtitle + type field or so;
+ * also, give different  subtitle formats really different types */
+static GstStaticCaps tmp_caps =
+GST_STATIC_CAPS ("application/x-subtitle-tmplayer");
 static GstStaticCaps smi_caps = GST_STATIC_CAPS ("application/x-subtitle-sami");
 static GstStaticCaps sub_caps = GST_STATIC_CAPS ("application/x-subtitle");
 
 #define SUB_CAPS (gst_static_caps_get (&sub_caps))
 #define SAMI_CAPS (gst_static_caps_get (&smi_caps))
+#define TMP_CAPS (gst_static_caps_get (&tmp_caps))
 
 static void
 gst_subparse_type_find (GstTypeFind * tf, gpointer private)
@@ -1000,6 +1161,10 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
       GST_DEBUG ("SAMI (time-based) format detected");
       caps = SAMI_CAPS;
       break;
+    case GST_SUB_PARSE_FORMAT_TMPLAYER:
+      GST_DEBUG ("TMPlayer (time based) format detected");
+      caps = TMP_CAPS;
+      break;
     default:
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
       GST_DEBUG ("no subtitle format detected");
@@ -1013,7 +1178,9 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  static gchar *sub_exts[] = { "srt", "sub", "mpsub", "mdvd", "smi", NULL };
+  static gchar *sub_exts[] = { "srt", "sub", "mpsub", "mdvd", "smi", "txt",
+    NULL
+  };
 
   GST_DEBUG_CATEGORY_INIT (sub_parse_debug, "subparse", 0, ".sub parser");
 
