@@ -185,7 +185,7 @@ uri_type_find (GstTypeFind * tf, gpointer unused)
 
 /*** application/xml **********************************************************/
 
-#define XML_BUFFER_SIZE 256
+#define XML_BUFFER_SIZE 16
 #define XML_INC_BUFFER {                                                \
   pos++;                                                                \
   if (pos == XML_BUFFER_SIZE) {                                         \
@@ -401,6 +401,29 @@ tta_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** audio/x-flac ***/
+static GstStaticCaps flac_caps = GST_STATIC_CAPS ("audio/x-flac");
+
+#define FLAC_CAPS (gst_static_caps_get(&flac_caps))
+
+static void
+flac_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data;
+
+  data = gst_type_find_peek (tf, 0, 5);
+  if (G_LIKELY (data)) {
+    /* standard flac */
+    if (memcmp (data, "fLaC", 4) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, FLAC_CAPS);
+    }
+    /* flac-in-ogg, see http://flac.sourceforge.net/ogg_mapping.html */
+    else if (memcmp (data, "\177FLAC\001", 6) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, FLAC_CAPS);
+    }
+  }
+}
+
 /*** audio/mpeg version 2, 4 ***/
 
 static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/mpeg, "
@@ -620,6 +643,7 @@ static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, "
 #define GST_MP3_TYPEFIND_TRY_HEADERS (5)
 #define GST_MP3_TYPEFIND_TRY_SYNC (GST_TYPE_FIND_MAXIMUM * 100) /* 10kB */
 #define GST_MP3_TYPEFIND_SYNC_SIZE (2048)
+#define GST_MP3_WRONG_HEADER (10)
 
 static void
 mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
@@ -631,6 +655,7 @@ mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
   guint64 skipped;
   gint last_free_offset = -1;
   gint last_free_framelen = -1;
+  gboolean headerstart = TRUE;
 
   *found_layer = 0;
   *found_prob = 0;
@@ -685,6 +710,10 @@ mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
             last_free_framelen = -1;
           }
 
+          /* Mark the fact that we didn't find a valid header at the beginning */
+          if (found == 0)
+            headerstart = FALSE;
+
           GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
               " (0x%" G_GINT64_MODIFIER "x) was not an mp3 header "
               "(possibly-free: %s)", found + 1, start_off + offset,
@@ -719,16 +748,19 @@ mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
             (GST_MP3_TYPEFIND_TRY_SYNC - skipped) /
             GST_MP3_TYPEFIND_TRY_HEADERS / GST_MP3_TYPEFIND_TRY_SYNC;
 
+        if (!headerstart
+            && probability > (GST_TYPE_FIND_MINIMUM + GST_MP3_WRONG_HEADER))
+          probability -= GST_MP3_WRONG_HEADER;
         if (probability < GST_TYPE_FIND_MINIMUM)
           probability = GST_TYPE_FIND_MINIMUM;
         if (start_off > 0)
           probability /= 2;
 
         GST_INFO
-            ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %u) / %u",
-            probability, GST_TYPE_FIND_MAXIMUM, found,
-            GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC, skipped,
-            GST_MP3_TYPEFIND_TRY_SYNC);
+            ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %"
+            G_GUINT64_FORMAT ") / %u", probability, GST_TYPE_FIND_MAXIMUM,
+            found, GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC,
+            (guint64) skipped, GST_MP3_TYPEFIND_TRY_SYNC);
         /* make sure we're not id3 tagged */
         head_data = gst_type_find_peek (tf, -128, 3);
         if (head_data && (memcmp (head_data, "TAG", 3) == 0)) {
@@ -906,7 +938,8 @@ wavpack_type_find (GstTypeFind * tf, gpointer unused)
       sublen += 1 + 1;          /* id + length */
     }
     if (sublen > blocksize - offset + 32) {
-      GST_LOG ("chunk length too big (%u > %u)", sublen, blocksize - offset);
+      GST_LOG ("chunk length too big (%u > %" G_GUINT64_FORMAT ")", sublen,
+          blocksize - offset);
       break;
     }
     if ((data[0] & 0x20) == 0) {
@@ -996,27 +1029,42 @@ static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
                                           (((guint8 *)(data))[3] == 0xC0) || \
                                           (((guint8 *)(data))[3] == 0xBD)))
 
+#define MPEG2_MAX_PROBE_LENGTH 4096     /* 4kB (random value) */
+
 static void
 mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 0, 5);
-  gint mpegversion;
+  guint8 *data, *data0;
+  gint mpegversion, len;
 
-  if (data && IS_MPEG_PACK_HEADER (data)) {
-    if ((data[4] & 0xC0) == 0x40) {
-      /* type 2 */
+  len = MPEG2_MAX_PROBE_LENGTH * 2;
+  do {
+    len = len / 2;
+    data = gst_type_find_peek (tf, 0, 5 + len);
+  } while (data == NULL && len >= 32);
+
+  if (!data)
+    return;
+
+  data0 = data;
+  while (len > 0) {
+    if (IS_MPEG_PACK_HEADER (data)) {
+      if ((data[4] & 0xC0) == 0x40) {
+        /* type 2 */
+        mpegversion = 2;
+        goto suggest;
+      } else if ((data[4] & 0xF0) == 0x20) {
+        mpegversion = 1;
+        goto suggest;
+      }
+    } else if (IS_MPEG_PES_HEADER (data)) {
+      /* PES stream */
       mpegversion = 2;
       goto suggest;
-    } else if ((data[4] & 0xF0) == 0x20) {
-      mpegversion = 1;
-      goto suggest;
     }
-  } else if (data && IS_MPEG_PES_HEADER (data)) {
-    /* PES stream */
-    mpegversion = 2;
-    goto suggest;
+    ++data;
+    --len;
   }
-
   return;
 suggest:
   {
@@ -1024,7 +1072,11 @@ suggest:
 
     gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
         G_TYPE_INT, mpegversion, NULL);
-    gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE, caps);
+    /* lower probability if the marker isn't right at the start */
+    if (data0 == data)
+      gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE, caps);
+    else
+      gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE - 10, caps);
     gst_caps_unref (caps);
   }
 };
@@ -1046,10 +1098,13 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
   }
 
   if (data[0] != 0 || data[1] != 0 || data[2] != 1) {
+    GST_LOG ("no sync");
     return 0;
   }
   offset += 4;
 
+  GST_LOG ("sync %02x", data[3]);
+
   switch (data[3]) {
     case 0xBA:                 /* pack header */
       data = gst_type_find_peek (tf, offset, 8);
@@ -1062,8 +1117,10 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
       if ((data[0] & 0xF1) != 0x21 ||
           (data[2] & 0x01) != 0x01 ||
           (data[4] & 0x01) != 0x01 ||
-          (data[5] & 0x80) != 0x80 || (data[7] & 0x01) != 0x01)
+          (data[5] & 0x80) != 0x80 || (data[7] & 0x01) != 0x01) {
+        GST_LOG ("wrong marker bits");
         return 0;
+      }
       break;
 
     case 0xB9:                 /* ISO end code */
@@ -1085,12 +1142,16 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
       }
       /* check marker bits */
       if ((data[0] & 0x80) != 0x80 ||
-          (data[2] & 0x01) != 0x01 || (data[4] & 0x20) != 0x20)
+          (data[2] & 0x01) != 0x01 || (data[4] & 0x20) != 0x20) {
+        GST_LOG ("wrong marker bits");
         return 0;
+      }
       /* check stream marker bits */
       for (offset = 6; offset < (size - 6); offset += 3) {
-        if (data[offset] <= 0xBB || (data[offset + 1] & 0xC0) != 0xC0)
+        if (data[offset] <= 0xBB || (data[offset + 1] & 0xC0) != 0xC0) {
+          GST_LOG ("wrong marker bits");
           return 0;
+        }
       }
       break;
 
@@ -1154,6 +1215,7 @@ mpeg1_sys_type_find (GstTypeFind * tf, gpointer unused)
       }
       g_assert (found <= GST_MPEG_TYPEFIND_TRY_HEADERS);
       if (found == GST_MPEG_TYPEFIND_TRY_HEADERS || packet_size == 1) {
+        GST_LOG ("suggesting mpeg1 system steeam");
         caps = gst_caps_copy (MPEG_SYS_CAPS);
         gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
             G_TYPE_INT, 1, NULL);
@@ -1272,19 +1334,41 @@ static GstStaticCaps mpeg4_video_caps = GST_STATIC_CAPS ("video/mpeg, "
 static void
 mpeg4_video_type_find (GstTypeFind * tf, gpointer unused)
 {
-  /* Header is a video object start code followed by a video object layer
-   * start code. The last byte of this 8-byte header can be from 0x20 - 0x2F */
-  static const guint8 header[] = { 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01 };
+  /* Header consists of: a series of start codes (00 00 01 xx), some with 
+   * associated data.
+   * Optionally, we start with a visual_object_sequence_start_code, followed by
+   * (optionally) visual_object_start_code), then the mandatory 
+   * video_object_start_code and video_object_layer_start_code)
+   */
   guint8 *data = NULL;
+  int offset = 0;
+  gboolean seen_vos = FALSE;
 
-  data = gst_type_find_peek (tf, 0, 8);
+  while (TRUE) {
+    data = gst_type_find_peek (tf, offset, 4);
+    if (data && data[0] == 0 && data[1] == 0 && data[2] == 1) {
+      int sc = data[3];
 
-  if (data && memcmp (data, header, 7) == 0 &&
-      data[7] >= 0x20 && data[7] <= 0x2F) {
-    GstCaps *caps = gst_caps_copy (MPEG4_VIDEO_CAPS);
+      if (sc == 0xB0)           /* visual_object_sequence_start_code */
+        offset += 5;
+      else if (sc == 0xB5)      /* visual_object_start_code */
+        offset += 5;
+      else if (sc >= 0x00 && sc <= 0x1F) {      /* video_object_start_code */
+        offset += 4;
+        seen_vos = TRUE;
+      } else if (sc >= 0x20 && sc <= 0x2F) {    /* video_object_layer_start_code */
+        if (seen_vos) {
+          GstCaps *caps = gst_caps_copy (MPEG4_VIDEO_CAPS);
 
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps);
-    gst_caps_unref (caps);
+          gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps);
+          gst_caps_unref (caps);
+          return;
+        } else
+          return;
+      } else
+        return;
+    } else
+      return;
   }
 }
 
@@ -1415,7 +1499,7 @@ aiff_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
-/*** audio/x-aiff ***/
+/*** audio/x-svx ***/
 
 static GstStaticCaps svx_caps = GST_STATIC_CAPS ("audio/x-svx");
 
@@ -1600,6 +1684,61 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
 };
 
 
+/*** image/x-quicktime ***/
+
+static GstStaticCaps qtif_caps = GST_STATIC_CAPS ("image/x-quicktime");
+
+#define QTIF_CAPS gst_static_caps_get(&qtif_caps)
+
+/* how many atoms we check before we give up */
+#define QTIF_MAXROUNDS 25
+
+static void
+qtif_type_find (GstTypeFind * tf, gpointer unused)
+{
+  const guint8 *data;
+  gboolean found_idsc = FALSE;
+  gboolean found_idat = FALSE;
+  guint64 offset = 0;
+  guint rounds = 0;
+
+  while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
+    guint64 size;
+
+    size = GST_READ_UINT32_BE (data);
+    if (size == 1) {
+      const guint8 *sizedata;
+
+      sizedata = gst_type_find_peek (tf, offset + 8, 8);
+      if (sizedata == NULL)
+        break;
+
+      size = GST_READ_UINT64_BE (sizedata);
+    }
+    if (size < 8)
+      break;
+
+    if (STRNCMP (data + 4, "idsc", 4) == 0)
+      found_idsc = TRUE;
+    if (STRNCMP (data + 4, "idat", 4) == 0)
+      found_idat = TRUE;
+
+    if (found_idsc && found_idat) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, QTIF_CAPS);
+      return;
+    }
+
+    offset += size;
+    if (++rounds > QTIF_MAXROUNDS)
+      break;
+  }
+
+  if (found_idsc || found_idat) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, QTIF_CAPS);
+    return;
+  }
+};
+
 /*** audio/x-mod ***/
 
 static GstStaticCaps mod_caps = GST_STATIC_CAPS ("audio/x-mod");
@@ -1953,7 +2092,7 @@ ogganx_type_find (GstTypeFind * tf, gpointer private)
 
     /* Check for an annodex fishbone header */
     data = gst_type_find_peek (tf, 28, 8);
-    if (memcmp (data, "fishead\0", 8) == 0)
+    if (data && memcmp (data, "fishead\0", 8) == 0)
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGGANX_CAPS);
 
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
@@ -1985,7 +2124,7 @@ vorbis_type_find (GstTypeFind * tf, gpointer private)
       return;
     data++;
     /* 4 byte samplerate must be != 0 */
-    if (*((guint32 *) data) == 0)
+    if (GST_READ_UINT32_LE (data) == 0)
       return;
     data += 16;
     /* blocksize checks */
@@ -2227,6 +2366,29 @@ au_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+
+/*** video/x-nuv ***/
+
+/* NOTE: we cannot replace this function with TYPE_FIND_REGISTER_START_WITH,
+ * as it is only possible to register one typefind factory per 'name'
+ * (which is in this case the caps), and the first one would be replaced by
+ * the second one. */
+static GstStaticCaps nuv_caps = GST_STATIC_CAPS ("video/x-nuv");
+
+#define NUV_CAPS (gst_static_caps_get(&nuv_caps))
+static void
+nuv_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 11);
+
+  if (data) {
+    if (memcmp (data, "MythTVVideo", 11) == 0
+        || memcmp (data, "NuppelVideo", 11) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, NUV_CAPS);
+    }
+  }
+}
+
 /*** audio/x-paris ***/
 /* NOTE: do not replace this function with two TYPE_FIND_REGISTER_START_WITH */
 static GstStaticCaps paris_caps = GST_STATIC_CAPS ("audio/x-paris");
@@ -2283,6 +2445,96 @@ msdos_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/x-mmsh ***/
+
+static GstStaticCaps mmsh_caps = GST_STATIC_CAPS ("application/x-mmsh");
+
+#define MMSH_CAPS gst_static_caps_get(&mmsh_caps)
+
+/* This is to recognise mssh-over-http */
+static void
+mmsh_type_find (GstTypeFind * tf, gpointer unused)
+{
+  static const guint8 asf_marker[16] = { 0x30, 0x26, 0xb2, 0x75, 0x8e, 0x66,
+    0xcf, 0x11, 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c
+  };
+
+  guint8 *data;
+
+  data = gst_type_find_peek (tf, 0, 2 + 2 + 4 + 2 + 2 + 16);
+  if (data && data[0] == 0x24 && data[1] == 0x48 &&
+      GST_READ_UINT16_LE (data + 2) > 2 + 2 + 4 + 2 + 2 + 16 &&
+      memcmp (data + 2 + 2 + 4 + 2 + 2, asf_marker, 16) == 0) {
+    GstCaps *caps = gst_caps_copy (MMSH_CAPS);
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+    gst_caps_unref (caps);
+    return;
+  }
+}
+
+/*** video/x-dirac ***/
+
+/* NOTE: we cannot replace this function with TYPE_FIND_REGISTER_START_WITH,
+ * as it is only possible to register one typefind factory per 'name'
+ * (which is in this case the caps), and the first one would be replaced by
+ * the second one. */
+static GstStaticCaps dirac_caps = GST_STATIC_CAPS ("video/x-dirac");
+
+#define DIRAC_CAPS (gst_static_caps_get(&dirac_caps))
+static void
+dirac_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 8);
+
+  if (data) {
+    if (memcmp (data, "BBCD", 4) == 0 || memcmp (data, "KW-DIRAC", 8) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, DIRAC_CAPS);
+    }
+  }
+}
+
+/*** video/vivo ***/
+
+static GstStaticCaps vivo_caps = GST_STATIC_CAPS ("video/vivo");
+
+#define VIVO_CAPS gst_static_caps_get(&vivo_caps)
+
+static void
+vivo_type_find (GstTypeFind * tf, gpointer unused)
+{
+  static const guint8 vivo_marker[] = { 'V', 'e', 'r', 's', 'i', 'o', 'n',
+    ':', 'V', 'i', 'v', 'o', '/'
+  };
+  guint8 *data;
+  guint hdr_len, pos;
+
+  data = gst_type_find_peek (tf, 0, 1024);
+  if (data == NULL || data[0] != 0x00)
+    return;
+
+  if ((data[1] & 0x80)) {
+    if ((data[2] & 0x80))
+      return;
+    hdr_len = ((guint) (data[1] & 0x7f)) << 7;
+    hdr_len += data[2];
+    if (hdr_len > 2048)
+      return;
+    pos = 3;
+  } else {
+    hdr_len = data[1];
+    pos = 2;
+  }
+
+  /* 1008 = 1022 - strlen ("Version:Vivo/") - 1 */
+  while (pos < 1008 && data[pos] == '\r' && data[pos + 1] == '\n')
+    pos += 2;
+
+  if (memcmp (data + pos, vivo_marker, sizeof (vivo_marker)) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, VIVO_CAPS);
+  }
+}
+
 /*** generic typefind for streams that have some data at a specific position***/
 typedef struct
 {
@@ -2361,6 +2613,8 @@ G_BEGIN_DECLS{                                                          \
   }                                                                     \
 }G_END_DECLS
 
+
+
 /*** plugin initialization ***/
 
 #define TYPE_FIND_REGISTER(plugin,name,rank,func,ext,caps,priv,notify) \
@@ -2400,6 +2654,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *mpeg_ts_exts[] = { "ts", NULL };
   static gchar *ogg_exts[] = { "anx", "ogg", "ogm", NULL };
   static gchar *qt_exts[] = { "mov", NULL };
+  static gchar *qtif_exts[] = { "qif", "qtif", "qti", NULL };
   static gchar *rm_exts[] = { "ra", "ram", "rm", "rmvb", NULL };
   static gchar *swf_exts[] = { "swf", "swfl", NULL };
   static gchar *utf8_exts[] = { "txt", NULL };
@@ -2452,6 +2707,8 @@ plugin_init (GstPlugin * plugin)
   };
   static gchar *flv_exts[] = { "flv", NULL };
   static gchar *m4v_exts[] = { "m4v", NULL };
+  static gchar *nuv_exts[] = { "nuv", NULL };
+  static gchar *vivo_exts[] = { "viv", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
       GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
@@ -2472,8 +2729,6 @@ plugin_init (GstPlugin * plugin)
   TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-vcd", GST_RANK_PRIMARY,
       cdxa_exts, "\000\377\377\377\377\377\377\377\377\377\377\000", 12,
       GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-flac", GST_RANK_PRIMARY,
-      flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/x-fli", GST_RANK_MARGINAL, flx_type_find,
       flx_exts, FLX_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-id3v2", GST_RANK_PRIMARY + 3,
@@ -2505,6 +2760,8 @@ plugin_init (GstPlugin * plugin)
       NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg4", GST_RANK_PRIMARY,
       mpeg4_video_type_find, m4v_exts, MPEG_VIDEO_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/x-nuv", GST_RANK_SECONDARY,
+      nuv_type_find, nuv_exts, NUV_CAPS, NULL, NULL);
 
   /* ISO formats */
   TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
@@ -2513,6 +2770,8 @@ plugin_init (GstPlugin * plugin)
       q3gp_type_find, q3gp_exts, Q3GP_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "video/quicktime", GST_RANK_SECONDARY,
       qt_type_find, qt_exts, QT_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "image/x-quicktime", GST_RANK_SECONDARY,
+      qtif_type_find, qtif_exts, QTIF_CAPS, NULL, NULL);
 
   TYPE_FIND_REGISTER (plugin, "text/html", GST_RANK_SECONDARY, html_type_find,
       html_exts, HTML_CAPS, NULL, NULL);
@@ -2598,6 +2857,8 @@ plugin_init (GstPlugin * plugin)
       zip_exts, "PK\003\004", 4, GST_TYPE_FIND_LIKELY);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-compress",
       GST_RANK_SECONDARY, compress_exts, "\037\235", 2, GST_TYPE_FIND_LIKELY);
+  TYPE_FIND_REGISTER (plugin, "audio/x-flac", GST_RANK_PRIMARY,
+      flac_type_find, flac_exts, FLAC_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-vorbis", GST_RANK_PRIMARY,
       vorbis_type_find, NULL, VORBIS_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "video/x-theora", GST_RANK_PRIMARY,
@@ -2633,14 +2894,18 @@ plugin_init (GstPlugin * plugin)
       ar_type_find, ar_exts, AR_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ms-dos-executable",
       GST_RANK_SECONDARY, msdos_type_find, msdos_exts, MSDOS_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/x-dirac", GST_RANK_PRIMARY,
+      dirac_type_find, NULL, DIRAC_CAPS, NULL, NULL);
 #if 0
   TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
       GST_RANK_PRIMARY, NULL, "BBCD", 4, GST_TYPE_FIND_MAXIMUM);
 #endif
-  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
-      GST_RANK_PRIMARY, NULL, "KW-DIRAC", 8, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "multipart/x-mixed-replace", GST_RANK_SECONDARY,
       multipart_type_find, NULL, MULTIPART_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-mmsh", GST_RANK_SECONDARY,
+      mmsh_type_find, NULL, MMSH_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/vivo", GST_RANK_SECONDARY,
+      vivo_type_find, vivo_exts, VIVO_CAPS, NULL, NULL);
   return TRUE;
 }
 
