@@ -164,7 +164,8 @@ enum
   PROP_DISPLAY,
   PROP_SYNCHRONOUS,
   PROP_PIXEL_ASPECT_RATIO,
-  PROP_FORCE_ASPECT_RATIO
+  PROP_FORCE_ASPECT_RATIO,
+  PROP_HANDLE_EVENTS
       /* FILL ME */
 };
 
@@ -201,7 +202,7 @@ gst_ximage_buffer_finalize (GstXImageBuffer * ximage)
   g_return_if_fail (ximage != NULL);
 
   ximagesink = ximage->ximagesink;
-  if (!ximagesink) {
+  if (G_UNLIKELY (ximagesink == NULL)) {
     GST_WARNING_OBJECT (ximagesink, "no sink found");
     goto beach;
   }
@@ -300,7 +301,7 @@ gst_ximagesink_handle_xerror (Display * display, XErrorEvent * xevent)
   char error_msg[1024];
 
   XGetErrorText (display, xevent->error_code, error_msg, 1024);
-  GST_DEBUG ("ximagesink failed to use XShm calls. error: %s", error_msg);
+  GST_DEBUG ("ximagesink triggered an XError. error: %s", error_msg);
   error_caught = TRUE;
   return 0;
 }
@@ -345,7 +346,8 @@ gst_ximagesink_check_xshm_calls (GstXImageSink * ximagesink,
 
   SHMInfo.shmid = shmget (IPC_PRIVATE, size, IPC_CREAT | 0777);
   if (SHMInfo.shmid == -1) {
-    GST_WARNING ("could not get shared memory of %d bytes", size);
+    GST_WARNING ("could not get shared memory of %" G_GSIZE_FORMAT " bytes",
+        size);
     goto beach;
   }
 
@@ -404,6 +406,7 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
   GstXImageBuffer *ximage = NULL;
   GstStructure *structure = NULL;
   gboolean succeeded = FALSE;
+  int (*handler) (Display *, XErrorEvent *);
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
@@ -421,14 +424,22 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
 
   g_mutex_lock (ximagesink->x_lock);
 
+  /* Setting an error handler to catch failure */
+  error_caught = FALSE;
+  handler = XSetErrorHandler (gst_ximagesink_handle_xerror);
+
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
     ximage->ximage = XShmCreateImage (ximagesink->xcontext->disp,
         ximagesink->xcontext->visual,
         ximagesink->xcontext->depth,
         ZPixmap, NULL, &ximage->SHMInfo, ximage->width, ximage->height);
-    if (!ximage->ximage) {
+    if (!ximage->ximage || error_caught) {
       g_mutex_unlock (ximagesink->x_lock);
+      /* Reset error handler */
+      error_caught = FALSE;
+      XSetErrorHandler (handler);
+      /* Push an error */
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
               ximage->width, ximage->height),
@@ -439,7 +450,8 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
 
     /* we have to use the returned bytes_per_line for our shm size */
     ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
-    GST_LOG_OBJECT (ximagesink, "XShm image size is %d, width %d, stride %d",
+    GST_LOG_OBJECT (ximagesink,
+        "XShm image size is %" G_GSIZE_FORMAT ", width %d, stride %d",
         ximage->size, ximage->width, ximage->ximage->bytes_per_line);
 
     ximage->SHMInfo.shmid = shmget (IPC_PRIVATE, ximage->size,
@@ -449,7 +461,8 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
               ximage->width, ximage->height),
-          ("could not get shared memory of %d bytes", ximage->size));
+          ("could not get shared memory of %" G_GSIZE_FORMAT " bytes",
+              ximage->size));
       goto beach;
     }
 
@@ -490,8 +503,12 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
         ximagesink->xcontext->depth,
         ZPixmap, 0, NULL,
         ximage->width, ximage->height, ximagesink->xcontext->bpp, 0);
-    if (!ximage->ximage) {
+    if (!ximage->ximage || error_caught) {
       g_mutex_unlock (ximagesink->x_lock);
+      /* Reset error handler */
+      error_caught = FALSE;
+      XSetErrorHandler (handler);
+      /* Push an error */
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
               ximage->width, ximage->height),
@@ -506,6 +523,11 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
 
     XSync (ximagesink->xcontext->disp, FALSE);
   }
+
+  /* Reset error handler */
+  error_caught = FALSE;
+  XSetErrorHandler (handler);
+
   succeeded = TRUE;
 
   GST_BUFFER_DATA (ximage) = (guchar *) ximage->ximage->data;
@@ -532,14 +554,14 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
   g_return_if_fail (ximage != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
+  /* Hold the object lock to ensure the XContext doesn't disappear */
+  GST_OBJECT_LOCK (ximagesink);
+
   /* If the destroyed image is the current one we destroy our reference too */
   if (ximagesink->cur_image == ximage) {
     ximagesink->cur_image = NULL;
   }
 
-  /* Hold the object lock to ensure the XContext doesn't disappear */
-  GST_OBJECT_LOCK (ximagesink);
-
   /* We might have some buffers destroyed after changing state to NULL */
   if (!ximagesink->xcontext) {
     GST_DEBUG_OBJECT (ximagesink, "Destroying XImage after XContext");
@@ -628,6 +650,7 @@ static void
 gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
 {
   GstVideoRectangle src, dst, result;
+  gboolean draw_border = FALSE;
 
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
@@ -635,6 +658,17 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
      concurrently from the data flow thread */
   g_mutex_lock (ximagesink->flow_lock);
 
+  if (G_UNLIKELY (ximagesink->xwindow == NULL)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return;
+  }
+
+  /* Draw borders when displaying the first frame. After this
+     draw borders only on expose event. */
+  if (!ximagesink->cur_image) {
+    draw_border = TRUE;
+  }
+
   /* Store a reference to the last image we put, lose the previous one */
   if (ximage && ximagesink->cur_image != ximage) {
     if (ximagesink->cur_image) {
@@ -648,6 +682,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
 
   /* Expose sends a NULL image, we take the latest frame */
   if (!ximage) {
+    draw_border = TRUE;
     if (ximagesink->cur_image) {
       ximage = ximagesink->cur_image;
     } else {
@@ -667,8 +702,10 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
 
   g_mutex_lock (ximagesink->x_lock);
 
-  gst_ximagesink_xwindow_draw_borders (ximagesink, ximagesink->xwindow, result);
-
+  if (draw_border) {
+    gst_ximagesink_xwindow_draw_borders (ximagesink, ximagesink->xwindow,
+        result);
+  }
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
     GST_LOG_OBJECT (ximagesink,
@@ -758,9 +795,11 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
      ConfigureNotify. This takes away flickering of video when resizing. */
   XSetWindowBackgroundPixmap (ximagesink->xcontext->disp, xwindow->win, None);
 
-  XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
-      StructureNotifyMask | PointerMotionMask | KeyPressMask |
-      KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+  if (ximagesink->handle_events) {
+    XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
+        StructureNotifyMask | PointerMotionMask | KeyPressMask |
+        KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+  }
 
   xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win,
       0, &values);
@@ -859,10 +898,12 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
 
     /* Then we get all pointer motion events, only the last position is
        interesting. */
+    g_mutex_lock (ximagesink->flow_lock);
     g_mutex_lock (ximagesink->x_lock);
     while (XCheckWindowEvent (ximagesink->xcontext->disp,
             ximagesink->xwindow->win, PointerMotionMask, &e)) {
       g_mutex_unlock (ximagesink->x_lock);
+      g_mutex_unlock (ximagesink->flow_lock);
 
       switch (e.type) {
         case MotionNotify:
@@ -873,10 +914,11 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         default:
           break;
       }
-
+      g_mutex_lock (ximagesink->flow_lock);
       g_mutex_lock (ximagesink->x_lock);
     }
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
     if (pointer_moved) {
       GST_DEBUG ("ximagesink pointer moved over window at %d,%d",
@@ -887,6 +929,7 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
   }
 
   /* We get all remaining events on our window to throw them upstream */
+  g_mutex_lock (ximagesink->flow_lock);
   g_mutex_lock (ximagesink->x_lock);
   while (XCheckWindowEvent (ximagesink->xcontext->disp,
           ximagesink->xwindow->win,
@@ -896,6 +939,7 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
 
     /* We lock only for the X function call */
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
     switch (e.type) {
       case ButtonPress:
@@ -918,12 +962,19 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
            events for interactivity/navigation */
         GST_DEBUG ("ximagesink key %d pressed over window at %d,%d",
             e.xkey.keycode, e.xkey.x, e.xkey.x);
+        g_mutex_lock (ximagesink->x_lock);
         keysym = XKeycodeToKeysym (ximagesink->xcontext->disp,
             e.xkey.keycode, 0);
+        g_mutex_unlock (ximagesink->x_lock);
         if (keysym != NoSymbol) {
+          char *key_str = NULL;
+
+          g_mutex_lock (ximagesink->x_lock);
+          key_str = XKeysymToString (keysym);
+          g_mutex_unlock (ximagesink->x_lock);
           gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
-              e.type == KeyPress ?
-              "key-press" : "key-release", XKeysymToString (keysym));
+              e.type == KeyPress ? "key-press" : "key-release", key_str);
+
         } else {
           gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
               e.type == KeyPress ? "key-press" : "key-release", "unknown");
@@ -933,17 +984,21 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         GST_DEBUG_OBJECT (ximagesink, "ximagesink unhandled X event (%d)",
             e.type);
     }
+    g_mutex_lock (ximagesink->flow_lock);
     g_mutex_lock (ximagesink->x_lock);
   }
   g_mutex_unlock (ximagesink->x_lock);
+  g_mutex_unlock (ximagesink->flow_lock);
 
   {
     gboolean exposed = FALSE;
 
+    g_mutex_lock (ximagesink->flow_lock);
     g_mutex_lock (ximagesink->x_lock);
     while (XCheckWindowEvent (ximagesink->xcontext->disp,
             ximagesink->xwindow->win, ExposureMask, &e)) {
       g_mutex_unlock (ximagesink->x_lock);
+      g_mutex_unlock (ximagesink->flow_lock);
 
       switch (e.type) {
         case Expose:
@@ -952,10 +1007,11 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         default:
           break;
       }
-
+      g_mutex_lock (ximagesink->flow_lock);
       g_mutex_lock (ximagesink->x_lock);
     }
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
     if (exposed) {
       gst_ximagesink_expose (GST_X_OVERLAY (ximagesink));
@@ -983,7 +1039,7 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
 static void
 gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
 {
-  gint par[][2] = {
+  static const gint par[][2] = {
     {1, 1},                     /* regular screen */
     {16, 15},                   /* PAL TV */
     {11, 10},                   /* 525 line Rec.601 video */
@@ -1283,6 +1339,7 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   GST_DEBUG_OBJECT (ximagesink, "intersection returned %" GST_PTR_FORMAT,
       intersection);
   if (gst_caps_is_empty (intersection)) {
+    gst_caps_unref (intersection);
     return FALSE;
   }
 
@@ -1318,17 +1375,28 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   ximagesink->fps_d = gst_value_get_fraction_denominator (fps);
 
   /* Notify application to set xwindow id now */
+  g_mutex_lock (ximagesink->flow_lock);
   if (!ximagesink->xwindow) {
+    g_mutex_unlock (ximagesink->flow_lock);
     gst_x_overlay_prepare_xwindow_id (GST_X_OVERLAY (ximagesink));
+  } else {
+    g_mutex_unlock (ximagesink->flow_lock);
   }
 
   /* Creating our window and our image */
-  g_assert (GST_VIDEO_SINK_WIDTH (ximagesink) > 0);
-  g_assert (GST_VIDEO_SINK_HEIGHT (ximagesink) > 0);
+  if (GST_VIDEO_SINK_WIDTH (ximagesink) <= 0 ||
+      GST_VIDEO_SINK_HEIGHT (ximagesink) <= 0) {
+    GST_ELEMENT_ERROR (ximagesink, CORE, NEGOTIATION, (NULL),
+        ("Invalid image size."));
+    return FALSE;
+  }
+
+  g_mutex_lock (ximagesink->flow_lock);
   if (!ximagesink->xwindow) {
     ximagesink->xwindow = gst_ximagesink_xwindow_new (ximagesink,
         GST_VIDEO_SINK_WIDTH (ximagesink), GST_VIDEO_SINK_HEIGHT (ximagesink));
   }
+  g_mutex_unlock (ximagesink->flow_lock);
 
   /* If our ximage has changed we destroy it, next chain iteration will create
      a new one */
@@ -1385,6 +1453,10 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
       g_mutex_unlock (ximagesink->x_lock);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      g_mutex_lock (ximagesink->flow_lock);
+      if (ximagesink->xwindow)
+        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
+      g_mutex_unlock (ximagesink->flow_lock);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1400,8 +1472,6 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      if (ximagesink->xwindow)
-        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->fps_n = 0;
       ximagesink->fps_d = 1;
       GST_VIDEO_SINK_WIDTH (ximagesink) = 0;
@@ -1423,10 +1493,13 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
 
       gst_ximagesink_bufferpool_clear (ximagesink);
 
+      g_mutex_lock (ximagesink->flow_lock);
       if (ximagesink->xwindow) {
+        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
         gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
         ximagesink->xwindow = NULL;
       }
+      g_mutex_unlock (ximagesink->flow_lock);
 
       gst_ximagesink_xcontext_clear (ximagesink);
       break;
@@ -1452,7 +1525,9 @@ gst_ximagesink_get_times (GstBaseSink * bsink, GstBuffer * buf,
       *end = *start + GST_BUFFER_DURATION (buf);
     } else {
       if (ximagesink->fps_n > 0) {
-        *end = *start + (GST_SECOND * ximagesink->fps_d) / ximagesink->fps_n;
+        *end = *start +
+            gst_util_uint64_scale_int (GST_SECOND, ximagesink->fps_d,
+            ximagesink->fps_n);
       }
     }
   }
@@ -1755,25 +1830,25 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
   GstXWindow *xwindow = NULL;
   XWindowAttributes attr;
 
-  /* If we already use that window return */
-  if (ximagesink->xwindow && (xwindow_id == ximagesink->xwindow->win))
-    return;
-
-  /* If the element has not initialized the X11 context try to do so */
-  if (!ximagesink->xcontext)
-    ximagesink->xcontext = gst_ximagesink_xcontext_get (ximagesink);
-
-  if (!ximagesink->xcontext) {
-    GST_WARNING_OBJECT (ximagesink,
-        "ximagesink was unable to obtain the X11 context.");
-    return;
-  }
-
   /* We acquire the stream lock while setting this window in the element.
      We are basically cleaning tons of stuff replacing the old window, putting
      images while we do that would surely crash */
   g_mutex_lock (ximagesink->flow_lock);
 
+  /* If we already use that window return */
+  if (ximagesink->xwindow && (xwindow_id == ximagesink->xwindow->win)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return;
+  }
+
+  /* If the element has not initialized the X11 context try to do so */
+  if (!ximagesink->xcontext &&
+      !(ximagesink->xcontext = gst_ximagesink_xcontext_get (ximagesink))) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    /* we have thrown a GST_ELEMENT_ERROR now */
+    return;
+  }
+
   /* If a window is there already we destroy it */
   if (ximagesink->xwindow) {
     gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
@@ -1801,9 +1876,11 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     xwindow->width = attr.width;
     xwindow->height = attr.height;
     xwindow->internal = FALSE;
-    XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
-        StructureNotifyMask | PointerMotionMask | KeyPressMask |
-        KeyReleaseMask);
+    if (ximagesink->handle_events) {
+      XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
+          StructureNotifyMask | PointerMotionMask | KeyPressMask |
+          KeyReleaseMask);
+    }
 
     xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win, 0, NULL);
     g_mutex_unlock (ximagesink->x_lock);
@@ -1820,17 +1897,51 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
 
-  if (!ximagesink->xwindow)
-    return;
-
   gst_ximagesink_ximage_put (ximagesink, NULL);
 }
 
+static void
+gst_ximagesink_set_event_handling (GstXOverlay * overlay,
+    gboolean handle_events)
+{
+  GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
+
+  ximagesink->handle_events = handle_events;
+
+  g_mutex_lock (ximagesink->flow_lock);
+
+  if (G_UNLIKELY (!ximagesink->xwindow)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return;
+  }
+
+  g_mutex_lock (ximagesink->x_lock);
+
+  if (handle_events) {
+    if (ximagesink->xwindow->internal) {
+      XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win,
+          ExposureMask | StructureNotifyMask | PointerMotionMask |
+          KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+    } else {
+      XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win,
+          ExposureMask | StructureNotifyMask | PointerMotionMask |
+          KeyPressMask | KeyReleaseMask);
+    }
+  } else {
+    XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win, 0);
+  }
+
+  g_mutex_unlock (ximagesink->x_lock);
+
+  g_mutex_unlock (ximagesink->flow_lock);
+}
+
 static void
 gst_ximagesink_xoverlay_init (GstXOverlayClass * iface)
 {
   iface->set_xwindow_id = gst_ximagesink_set_xwindow_id;
   iface->expose = gst_ximagesink_expose;
+  iface->handle_events = gst_ximagesink_set_event_handling;
 }
 
 /* =========================================== */
@@ -1886,6 +1997,10 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
       }
     }
       break;
+    case PROP_HANDLE_EVENTS:
+      gst_ximagesink_set_event_handling (GST_X_OVERLAY (ximagesink),
+          g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1916,6 +2031,9 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
       if (ximagesink->par)
         g_value_transform (ximagesink->par, value);
       break;
+    case PROP_HANDLE_EVENTS:
+      g_value_set_boolean (value, ximagesink->handle_events);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1978,6 +2096,7 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
 
   ximagesink->synchronous = FALSE;
   ximagesink->keep_aspect = FALSE;
+  ximagesink->handle_events = TRUE;
 }
 
 static void
@@ -2022,6 +2141,10 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_PIXEL_ASPECT_RATIO,
       g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
           "The pixel aspect ratio of the device", "1/1", G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_HANDLE_EVENTS,
+      g_param_spec_boolean ("handle-events", "Handle XEvents",
+          "When enabled, XEvents will be selected and handled", TRUE,
+          G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_ximagesink_change_state;
 
@@ -2089,9 +2212,10 @@ gst_ximagesink_get_type (void)
     g_type_add_interface_static (ximagesink_type, GST_TYPE_X_OVERLAY,
         &overlay_info);
 
-    /* register type in a more safe place instead of at runtime since the
-     * type registration is not threadsafe. */
-    gst_ximage_buffer_get_type ();
+    /* register type and create class in a more safe place instead of at
+     * runtime since the type registration and class creation is not
+     * threadsafe. */
+    g_type_class_ref (gst_ximage_buffer_get_type ());
   }
 
   return ximagesink_type;
