@@ -110,6 +110,8 @@ cleanup_videorate (GstElement * videorate)
 
   gst_element_set_state (videorate, GST_STATE_NULL);
   gst_element_get_state (videorate, NULL, NULL, GST_CLOCK_TIME_NONE);
+  gst_pad_set_active (mysrcpad, FALSE);
+  gst_pad_set_active (mysinkpad, FALSE);
   gst_check_teardown_src_pad (videorate);
   gst_check_teardown_sink_pad (videorate);
   gst_check_teardown_element (videorate);
@@ -118,7 +120,7 @@ cleanup_videorate (GstElement * videorate)
 GST_START_TEST (test_one)
 {
   GstElement *videorate;
-  GstBuffer *inbuffer, *outbuffer;
+  GstBuffer *inbuffer;
   GstCaps *caps;
 
   videorate = setup_videorate ();
@@ -259,7 +261,6 @@ GST_START_TEST (test_wrong_order_from_zero)
   GstElement *videorate;
   GstBuffer *first, *second, *third, *outbuffer;
   GstCaps *caps;
-  guint64 in, out, dropped, duplicated;
 
   videorate = setup_videorate ();
   fail_unless (gst_element_set_state (videorate,
@@ -349,7 +350,6 @@ GST_START_TEST (test_wrong_order)
   GstElement *videorate;
   GstBuffer *first, *second, *third, *fourth, *outbuffer;
   GstCaps *caps;
-  guint64 in, out, dropped, duplicated;
 
   videorate = setup_videorate ();
   fail_unless (gst_element_set_state (videorate,
@@ -458,7 +458,7 @@ GST_END_TEST;
 GST_START_TEST (test_no_framerate)
 {
   GstElement *videorate;
-  GstBuffer *inbuffer, *outbuffer;
+  GstBuffer *inbuffer;
   GstCaps *caps;
 
   videorate = setup_videorate ();
